#include "Paper.h"

int xAudio::Init()
{
	if(!m_hOpenALDLL)
	{
		m_hOpenALDLL = LoadLibrary("openal32.dll");

		if(!m_hOpenALDLL)
		{
			MessageBox(NULL, "Error loading OpenAL32.dll - Running OpenAL Installer", "Error", MB_OK);
			strcpy(g_xPaperAPI.m_strExitCommand, "oalinst.exe");
			g_xPaperAPI.m_bRunning = 0;
			return PE_RES_OPENALMUSICLOADFILEERROR;
		}

		// Initialize Function Pointers
		alGetError				= (LPALGETERROR)			GetProcAddress(m_hOpenALDLL, "alGetError");
		alGetEnumValue			= (LPALGETENUMVALUE)		GetProcAddress(m_hOpenALDLL, "alGetEnumValue");
		alGenBuffers			= (LPALGENBUFFERS)			GetProcAddress(m_hOpenALDLL, "alGenBuffers");
		alGenSources			= (LPALGENSOURCES)			GetProcAddress(m_hOpenALDLL, "alGenSources");
		alGetSourcei			= (LPALGETSOURCEI)			GetProcAddress(m_hOpenALDLL, "alGetSourcei");
		alGetSourcef			= (LPALGETSOURCEF)			GetProcAddress(m_hOpenALDLL, "alGetSourcef");
		alBufferData			= (LPALBUFFERDATA)			GetProcAddress(m_hOpenALDLL, "alBufferData");
		alSourceQueueBuffers	= (LPALSOURCEQUEUEBUFFERS)	GetProcAddress(m_hOpenALDLL, "alSourceQueueBuffers");
		alSourceUnqueueBuffers	= (LPALSOURCEUNQUEUEBUFFERS)GetProcAddress(m_hOpenALDLL, "alSourceUnqueueBuffers");
		alSourcei				= (LPALSOURCEI)				GetProcAddress(m_hOpenALDLL, "alSourcei");
		alSourcef				= (LPALSOURCEF)				GetProcAddress(m_hOpenALDLL, "alSourcef");
		alSourcePlay			= (LPALSOURCEPLAY)			GetProcAddress(m_hOpenALDLL, "alSourcePlay");
		alSourceStop			= (LPALSOURCESTOP)			GetProcAddress(m_hOpenALDLL, "alSourceStop");
		alSourcePause			= (LPALSOURCEPAUSE)			GetProcAddress(m_hOpenALDLL, "alSourcePause");
		alDeleteSources			= (LPALDELETESOURCES)		GetProcAddress(m_hOpenALDLL, "alDeleteSources");
		alDeleteBuffers			= (LPALDELETEBUFFERS)		GetProcAddress(m_hOpenALDLL, "alDeleteBuffers");
		alcGetString			= (LPALCGETSTRING)			GetProcAddress(m_hOpenALDLL, "alcGetString");
		alcOpenDevice			= (LPALCOPENDEVICE)			GetProcAddress(m_hOpenALDLL, "alcOpenDevice");
		alcCloseDevice			= (LPALCCLOSEDEVICE)		GetProcAddress(m_hOpenALDLL, "alcCloseDevice");
		alcCreateContext		= (LPALCCREATECONTEXT)		GetProcAddress(m_hOpenALDLL, "alcCreateContext");
		alcDestroyContext		= (LPALCDESTROYCONTEXT)		GetProcAddress(m_hOpenALDLL, "alcDestroyContext");
		alcMakeContextCurrent	= (LPALCMAKECONTEXTCURRENT)	GetProcAddress(m_hOpenALDLL, "alcMakeContextCurrent");
	}

	if(!m_pDevice)
	{
		m_pDevice = alcOpenDevice(alcGetString(NULL, ALC_DEFAULT_DEVICE_SPECIFIER));
		if(m_pDevice)
		{
			m_pContext = alcCreateContext(m_pDevice, NULL);
			if(m_pContext)
				alcMakeContextCurrent(m_pContext);
			else
				alcCloseDevice(m_pDevice);
		}
	}

	if(!m_pSources)
	{
		if(m_iMaxSourcesOverride == 0)
		{
			ALuint sources[PE_SOURCE_MAX];
			for(m_iMaxSources = 0; m_iMaxSources < PE_SOURCE_MAX; m_iMaxSources++)
			{
				alGenSources(1, &sources[m_iMaxSources]);

				if(alGetError() != AL_NO_ERROR)
					break;
			}

			m_pSources = new ALuint[m_iMaxSources];
			for(int i = 0; i < m_iMaxSources; i++)
				m_pSources[i] = sources[i];

			m_iMaxSourcesOverride = m_iMaxSources;
		}
		else
		{
			m_iMaxSources = m_iMaxSourcesOverride;
			m_pSources = new ALuint[m_iMaxSources];
			memset(m_pSources, 0, sizeof(ALuint) * m_iMaxSources);
			alGenSources(m_iMaxSources, m_pSources);
		}

		m_iNextSoundSource = PE_SOURCE_FIRSTSOUND;
	}

	return PE_RES_NOERROR;
}

void xAudio::Shutdown()
{
	alcMakeContextCurrent(NULL);

	if(m_pSources)
	{
		alDeleteSources(m_iMaxSources, m_pSources);
		delete [] m_pSources;
		m_pSources = NULL;
		m_iMaxSources = 0;
	}

	if(m_pContext)
	{
		alcDestroyContext(m_pContext);
		m_pContext = NULL;
	}

	if(m_pDevice)
	{
		alcCloseDevice(m_pDevice);
		m_pDevice = NULL;
	}

	if(m_hOpenALDLL)
	{
		FreeLibrary(m_hOpenALDLL);
		m_hOpenALDLL = NULL;
	}
}

xSound* xAudio::LoadSound(const char* strPathName)
{
	xSound *ret = m_xSoundPool.CreateNewObject(strPathName);

	if(!ret->m_bLoaded)
	{
		ret->m_bLoaded			= true;
		ret->alGetEnumValue		= alGetEnumValue;
		ret->alGenBuffers		= alGenBuffers;
		ret->alGetSourcei		= alGetSourcei;
		ret->alBufferData		= alBufferData;
		ret->alSourcei			= alSourcei;
		ret->alSourcef			= alSourcef;
		ret->alSourcePlay		= alSourcePlay;
		ret->alSourceStop		= alSourceStop;
		ret->alSourcePause		= alSourcePause;
		ret->alDeleteSources	= alDeleteSources;
		ret->alDeleteBuffers	= alDeleteBuffers;

		if(ret->Open(strPathName))
			return NULL;
	}

	ret->m_iReferenceCount++;

	return ret;
}

xSound* xAudio::LoadSoundWithRandom(const char* strPathName, bool bDontRepeat)
{
	int i = 2;
	char temp[256], temp2[256];

	xSound *ret = LoadSound(strPathName);

	if(ret)
	{
		strcpy(temp2, strPathName);
		temp2[strlen(temp2) - 4] = '\0';

		do
		{
			sprintf(temp, "%s%i.wav", temp2, i++);
		}
		while(!ret->AddVariant(temp));
	}

	ret->m_bDontRepeat = bDontRepeat;
	ret->m_pPlayedSounds = new int[ret->m_iNumSounds];
	ret->m_iCurrentPlayedIndex = 0;
	ret->m_iReferenceCount++;

	return ret;
}

xAudioStream* xAudio::LoadMusic(const char* strPathName, bool bLooping, bool bAnalyzing)
{
	xAudioStream *ret = new xAudioStream;

	ret->alGetEnumValue			= alGetEnumValue;
	ret->alGenBuffers			= alGenBuffers;
	ret->alGetSourcei			= alGetSourcei;
	ret->alGetSourcef			= alGetSourcef;
	ret->alBufferData			= alBufferData;
	ret->alSourcei				= alSourcei;
	ret->alSourcef				= alSourcef;
	ret->alSourceQueueBuffers	= alSourceQueueBuffers;
	ret->alSourceUnqueueBuffers	= alSourceUnqueueBuffers;
	ret->alSourcePlay			= alSourcePlay;
	ret->alSourceStop			= alSourceStop;
	ret->alDeleteSources		= alDeleteSources;
	ret->alDeleteBuffers		= alDeleteBuffers;

	if(ret->Open(strPathName, bLooping, bAnalyzing))
		return NULL;
			
	return ret;
}

void xAudio::UnloadSound(xSound* pSound)
{
	pSound->m_iReferenceCount--;

	if(pSound->m_iReferenceCount <= 0)
	{
		pSound->Release();
		m_xSoundPool.DeleteObject(pSound);
	}
}

void xAudio::UnloadMusic(xAudioStream *pMusic)
{
	pMusic->Release();

	delete pMusic;
}

ALuint xAudio::GetSource(bool bStream)
{
	xPoolItem<xSound> *sound;
	int state, i;

	if(bStream)
	{
		//DEBUGLOG("GetSource - bStream - %u\n", m_pSources[PE_SOURCE_MUSIC]);
		return m_pSources[PE_SOURCE_MUSIC];
	}

	//if(m_pSources[m_iNextSoundSource] != 0)
		//alDeleteSources(1, &m_pSources[m_iNextSoundSource]);

	//alGenSources(1, &m_pSources[m_iNextSoundSource]);

	for(i = PE_SOURCE_FIRSTSOUND; i < m_iMaxSources; i++)
	{
		alGetSourcei(m_pSources[i], AL_SOURCE_STATE, &state);
		if(state != AL_PLAYING)
		{
			for(sound = m_xSoundPool.m_xObjectList; sound; sound = sound->m_pNext)
				if(((xSound*)sound)->m_uiSource == m_pSources[i])
					((xSound*)sound)->m_xSoundState = PE_SOUNDSTATE_STOPPED;

			//DEBUGLOG("GetSource returning %u\n", m_pSources[i]);
			alSourceStop(m_pSources[i]);
			return m_pSources[i];
		}
	}

	// Leaving the old code path in as a backup
	alSourceStop(m_pSources[m_iNextSoundSource]);

	ALuint ret = m_pSources[m_iNextSoundSource];

	m_iNextSoundSource++;

	if(m_iNextSoundSource >= m_iMaxSources - 1)
		m_iNextSoundSource = PE_SOURCE_FIRSTSOUND;

	//DEBUGLOG("GetSource - %i - %u\n", m_iNextSoundSource, ret);

	for(sound = m_xSoundPool.m_xObjectList; sound; sound = sound->m_pNext)
		if(((xSound*)sound)->m_uiSource == m_pSources[i])
			((xSound*)sound)->m_xSoundState = PE_SOUNDSTATE_STOPPED;

	return ret;
}

void xAudio::StopAllSounds()
{
	xSound *currentsound = (xSound*)m_xSoundPool.m_xObjectList;
	while(currentsound)
	{
		currentsound->Stop();

		currentsound = (xSound*)currentsound->m_pNext;
	}
}
