#ifndef XAUDIOSTREAM_H_
#define XAUDIOSTREAM_H_

#include "openal/vorbisfile.h"
#include "xFileIO.h"

#define PE_OPENAL_OGG_NUMBUFFERS	4

enum xAudioStreamState
{
	PE_AUDIOSTREAMSTATE_NULL = 0,
	PE_AUDIOSTREAMSTATE_INITIALIZED,
	PE_AUDIOSTREAMSTATE_STOPPED,
	PE_AUDIOSTREAMSTATE_PLAYING
};

class xAudioStream
{
public:
	xAudioStream()
	{
		m_pDecodeBuffer		= NULL;
		m_bPlaying			= 0;
		m_bDone				= 0;
		m_bLooping			= 0;
		m_bAnalyzing		= 0;
	}

	int  Open(const char *strPathName, bool bLooping = false, bool bAnalyzing = false);
	int  Play();
	void ContinuePlaying();
	int  Stop();
	int  Release();

	//int Pause();
	//int UnPause();

	void  SetVolume(float fVolume = 1.0);
	DWORD GetTime();
	int   SetTime(DWORD dwMilliseconds);
	DWORD GetLength() { return m_dwLength; }
	bool  IsDone() { return m_bDone; }
	bool  IsPlaying() { return m_bPlaying; }

	unsigned long	GetChannelCount() { return m_ulChannels; }
	unsigned long	GetSampleRate() { return m_ulFrequency; }
	unsigned long	DecodeOggVorbis();
	char*			GetDecodeBuffer() { return m_pDecodeBuffer; }

	// OpenAL Function Pointers
	LPALGETENUMVALUE			alGetEnumValue;
	LPALGENBUFFERS				alGenBuffers;
	LPALGETSOURCEI				alGetSourcei;
	LPALGETSOURCEF				alGetSourcef;
	LPALBUFFERDATA				alBufferData;
	LPALSOURCEI					alSourcei;
	LPALSOURCEF					alSourcef;
	LPALSOURCEQUEUEBUFFERS		alSourceQueueBuffers;
	LPALSOURCEUNQUEUEBUFFERS	alSourceUnqueueBuffers;
	LPALSOURCEPLAY				alSourcePlay;
	LPALSOURCESTOP				alSourceStop;
	LPALDELETESOURCES			alDeleteSources;
	LPALDELETEBUFFERS			alDeleteBuffers;

	xAudioStreamState m_xAudioStreamState;

protected:
	bool			 m_bPlaying;
	bool			 m_bDone;
	bool			 m_bLooping;
	bool			 m_bAnalyzing;
	DWORD			 m_dwLength;
	xFileIO			 m_xFile;
	OggVorbis_File	 m_ovFile;
	vorbis_info		*m_pVorbisInfo;
	char			*m_pDecodeBuffer;
	unsigned long	 m_ulChannels;
	unsigned long	 m_ulFormat;
	unsigned long	 m_ulBufferSize;
	unsigned long	 m_ulFrequency;
	ALuint			 m_uiSource;
	ALuint			 m_uiBuffers[PE_OPENAL_OGG_NUMBUFFERS];
};

#endif
