#include "Paper.h"

int xFileIO::StartFindFiles(const char* strFilter, bool bDirectoriesOnly)
{
	m_bFindDirectoriesOnly = bDirectoriesOnly;

	m_hFindHandle = FindFirstFile(strFilter, &m_pFindData);

	while(bDirectoriesOnly && (!(m_pFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) || m_pFindData.cFileName[0] == '.') && m_hFindHandle != INVALID_HANDLE_VALUE)
		FindNextFile(m_hFindHandle, &m_pFindData);

	if(m_hFindHandle == INVALID_HANDLE_VALUE)
		m_bFindFilesComplete = true;
	else
		m_bFindFilesComplete = false;

	return PE_RES_NOERROR;
}

bool xFileIO::GetNextFile(char* strFileOut)
{
	if(m_bFindFilesComplete)
		return false;

	int i, j;
	int fullpathlength = strlen(m_pFindData.cFileName);

	for(i = fullpathlength - 1; i >= 0 ; i--)
		if(m_pFindData.cFileName[i] == '\\' || m_pFindData.cFileName[i] == '/')
			break;

	i++;

	for(j = 0; i < fullpathlength; i++, j++)
		strFileOut[j] = m_pFindData.cFileName[i];

	if(!FindNextFile(m_hFindHandle, &m_pFindData))
		m_bFindFilesComplete = true;

	while(m_bFindDirectoriesOnly && (!(m_pFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) || m_pFindData.cFileName[0] == '.') && m_hFindHandle != INVALID_HANDLE_VALUE)
	{
		if(!FindNextFile(m_hFindHandle, &m_pFindData))
		{
			m_bFindFilesComplete = true;
			break;
		}
	}

	return true;
}

int xFileIO::EndFindFiles()
{
	m_hFindHandle = NULL;

	return PE_RES_NOERROR;
}

int xFileIO::OpenFile(const char* strFileName, bool bReadOnly, bool bBinary)
{
	m_strPathName = strFileName;

	if(bReadOnly == true)
	{
		if(bBinary == true)
		{
			m_pFile = fopen(strFileName, "rb");
			if(m_pFile == NULL)
				return PE_RES_FILEDOESNOTEXIST;
		}
		else
		{
			m_pFile = fopen(strFileName, "r");
			if(m_pFile == NULL)
				return PE_RES_FILEDOESNOTEXIST;
		}
	}
	else
	{
		if(bBinary == true)
		{
			m_pFile = fopen(strFileName, "wb");
			if(m_pFile == NULL)
				return PE_RES_FILEOPENERROR;
		}
		else
		{
			m_pFile = fopen(strFileName, "w+");
			if(m_pFile == NULL)
				return PE_RES_FILEOPENERROR;
		}
	}

	return PE_RES_NOERROR;
}

int xFileIO::ReadData(void* pOutputData, long lSize, int *iReadSize)
{
	int readsize;

	if(iReadSize == NULL)
		iReadSize = &readsize;

	*iReadSize = 0;

	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	*iReadSize = fread(pOutputData, 1, lSize, m_pFile);

	if(*iReadSize < lSize)
		return PE_RES_FILEREADERROR;

	return PE_RES_NOERROR;
}

int xFileIO::ReadLine(char* pOutputData, int iMaxSize)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	if(fgets(pOutputData, iMaxSize, m_pFile) == NULL)
		return PE_RES_FILEREADERROR;

	//Remove '\n' from end of line
	pOutputData[strlen(pOutputData) - 1] = '\0';

	return PE_RES_NOERROR;
}

int xFileIO::ReadToEnd(void* pOutputData)
{
	long lSize = 0;

	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek (m_pFile, 0, SEEK_END);
	lSize = ftell(m_pFile);
	rewind(m_pFile);

	fread(pOutputData, 1, lSize, m_pFile);

	if(pOutputData == NULL)
		return PE_RES_FILEREADERROR;

	return PE_RES_NOERROR;
}

int xFileIO::SkipAhead(long lBytesToSkip)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek(m_pFile, lBytesToSkip, SEEK_CUR);

	return PE_RES_NOERROR;
}

int xFileIO::SetPosition(long lBytesOffset)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fseek(m_pFile, lBytesOffset, SEEK_SET);

	return PE_RES_NOERROR;
}

int xFileIO::WriteData(void* pInputData, long lSize)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fwrite(pInputData, 1, lSize, m_pFile);
	fflush(m_pFile);

	return PE_RES_NOERROR;
}

int xFileIO::WriteString(const char* strOutputText)
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fputs(strOutputText, m_pFile);
	fflush(m_pFile);

	return PE_RES_NOERROR;
}

long xFileIO::GetCurrentOffset()
{
	return ftell(m_pFile);
}

int xFileIO::CloseFile()
{
	if(m_pFile == NULL)
		return PE_RES_FILENOTOPEN;

	fclose(m_pFile);

	return PE_RES_NOERROR;
}

bool xFileIO::FileExists(const char *strPath)
{
	return GetFileAttributes(strPath) != INVALID_FILE_ATTRIBUTES;
}

bool xFileIO::DirectoryExists(const char *strPath)
{
	DWORD fileattributes = GetFileAttributes(strPath);

	if(fileattributes != INVALID_FILE_ATTRIBUTES)
		return fileattributes & FILE_ATTRIBUTE_DIRECTORY;

	return false;
}