/////////////////////////////////////////////////////////////////
// Name: xFileIO.h
/////////////////////////////////////////////////////////////////
// Description:
// - Represents an open file and/or file search
/////////////////////////////////////////////////////////////////
// - 7.6.2004, Update, Arto Ruotsalainen(Arex)
//   Comment stuff added to begin of the file.
//   Add new updates/fixes above this, so the old updates
//   will be at the bottom.
/////////////////////////////////////////////////////////////////
#ifndef XFILEIO_H_
#define XFILEIO_H_

#include <stdio.h>
#include "xResults.h"
#include "xList.h"

class xFileIO
{
public:
	int		StartFindFiles(const char* strFilter, bool bDirectoriesOnly = false);
	bool	GetNextFile(char* strFileOut);
	int		EndFindFiles();

	int		OpenFile(const char* strFileName, bool bReadOnly, bool bBinary);
	int		ReadData(void* pOutputData, long lSize, int *iReadSize = 0);
	int		ReadLine(char* pOutputData, int iMaxSize);
	int		ReadToEnd(void* pOutputData);
	int		SkipAhead(long lBytesToSkip);
	int		SetPosition(long lBytesOffset);
	int		WriteData(void* pInputData, long lSize);
	int		WriteString(const char* strOutputText);
	void*	GetFilePointer() { return m_pFile; }
	long	GetCurrentOffset();
	int		CloseFile();
	bool	IsOpen() { return m_pFile != NULL; }

	void		Shutdown() {}
	const char*	GetPathName() { return m_strPathName; }

	bool	FileExists(const char *strPath);
	bool	DirectoryExists(const char *strPath);

protected:
	WIN32_FIND_DATA	m_pFindData;
	HANDLE			m_hFindHandle;
	bool			m_bFindFilesComplete;
	FILE*			m_pFile;
	bool			m_bFindDirectoriesOnly;
	const char*		m_strPathName;
	int				m_iResult;
};
typedef xList<xFileIO> xFileIOList;

#endif