#include "Paper.h"

FORCE_INLINE void xGUIWindowSyncRenderables(xGUIWindow *pWindow, bool bRefresh = false);

int xGUIWindow::Init(xSceneGraph *pSceneGraph, xGUI *pGUI, xGUIWindowList *pMyWindowList, const char *strFile, float fDepth)
{
	m_pSceneGraph	= pSceneGraph;
	m_pGUI          = pGUI;
	m_pOwningWindow = pMyWindowList;

	xGUIObject::Init();

	m_pParser = new xParser;

	if(m_iResult = m_pParser->OpenScript(strFile))
		assert(0);

	char function[256], params[3][52];
	while(!m_pParser->ReadLine(function, params[0], params[1], params[2]))
	{
		if(!stricmp(function, "CreateWindow"))
		{
			if(m_iResult = LoadWindow(fDepth))
				assert(0);
		}
		else if(!stricmp(function, "AddChildObject"))
		{
			m_strThemeName = m_pParser->GetVariable("m_strThemeName");

			if(m_iResult = AddChildObject(params[0]))
				assert(0);
		}
		else if(!stricmp(function, "StartAnimation"))
			LoadAnimation();
	}

	m_pParser->EndScript();
	delete m_pParser;

	if(m_strPostLoadCommand)
		m_pGUI->pfnRunPostLoadCommand(m_strPostLoadCommand, m_strPostLoadParams, m_pOwningWindow);

	xGUIWindowSyncRenderables(this);

	return PE_RES_NOERROR;
}

void xGUIWindowPreUpdate(xGUIObject* pThis, DWORD dwTime)
{
	if(((xGUIWindow*)pThis)->GetHidden())
		return;

	if(((xGUIWindow*)pThis)->m_strKeyPressCommand)
		pThis->m_pGUI->pfnRunKeyCommand(((xGUIWindow*)pThis)->m_strKeyPressCommand, ((xGUIWindow*)pThis)->m_strKeyPressParams, pThis->m_pOwningWindow, pThis);

	static long lastmousex = g_xPaperAPI.m_xInput.m_lMouseX, lastmousey = g_xPaperAPI.m_xInput.m_lMouseY;
	//Check for mouse down and set 
	//m_bHasFocus and m_bDragging accordingly
	if(g_xPaperAPI.m_xInput.m_buttonMouseLeft.JustPressed(true) && !((xGUIWindow*)pThis)->GetHidden())
	{
		long mousex = g_xPaperAPI.m_xInput.m_lMouseX, mousey = g_xPaperAPI.m_xInput.m_lMouseY;
		if(mousex > pThis->GetX() && mousex < pThis->GetX() + pThis->GetWidth() &&
		   mousey > pThis->GetY() && mousey < pThis->GetY() + pThis->GetHeight())
		{
			//Temporary Close Button
			if(mousex > pThis->GetX() + ((xGUIWindow*)pThis)->m_fCloseButtonX && mousex < pThis->GetX() + ((xGUIWindow*)pThis)->m_fCloseButtonX + ((xGUIWindow*)pThis)->m_fCloseButtonWidth &&
			   mousey > pThis->GetY() + ((xGUIWindow*)pThis)->m_fCloseButtonY && mousey < pThis->GetY() + ((xGUIWindow*)pThis)->m_fCloseButtonY + ((xGUIWindow*)pThis)->m_fCloseButtonHeight &&
			   ((xGUIWindow*)pThis)->m_bClosable)
			{
			   ((xGUIWindow*)pThis)->Hide();
			}
			else
			{
				if(((xGUIWindow*)pThis)->m_bDragable)
					((xGUIWindow*)pThis)->m_bDragging = true;

				pThis->m_bHasFocus = true;
				lastmousex	= g_xPaperAPI.m_xInput.m_lMouseX;
				lastmousey	= g_xPaperAPI.m_xInput.m_lMouseY;
			}
		}
		else
		{
			pThis->m_bHasFocus = ((xGUIWindow*)pThis)->m_bAlwaysFocused;
			((xGUIWindow*)pThis)->m_bDragging = false;
		}
	}

	if(pThis->m_bHasFocus && g_xPaperAPI.m_xInput.m_keyTab.IsDown())
	{
		if(g_xPaperAPI.m_xInput.m_keyTab.JustPressed(true) || g_xPaperAPI.m_xInput.m_keyTab.ShouldRepeat(dwTime))
		{
			g_xPaperAPI.m_xInput.m_keyTab.Eaten();

			xGUIObjectList *focusedobj = NULL;
			for(focusedobj = pThis->m_xChildObjectList.m_pNext; focusedobj; focusedobj = focusedobj->m_pNext)
				if(focusedobj->m_pData->m_bHasFocus)
					break;

			xGUIObjectList *newfocusobj = ((xGUIWindow*)pThis)->GetNextTabIndexObject(focusedobj);
			if(newfocusobj)
			{
				focusedobj->m_pData->m_bHasFocus = false;
				newfocusobj->m_pData->m_bHasFocus = true;
			}
		}
	}
	else
	{
		xGUIObjectList *curobj = &pThis->m_xChildObjectList;
		while(curobj->m_pNext)
		{
			curobj = curobj->m_pNext;

			if(curobj->m_pData->pfnPreUpdate)
				curobj->m_pData->pfnPreUpdate(curobj->m_pData, dwTime);

			if(curobj->m_pData->m_bHasFocus)
				((xGUIWindow*)pThis)->m_bDragging = false;
		}
	}

	if(!g_xPaperAPI.m_xInput.m_buttonMouseLeft.IsDown())
		((xGUIWindow*)pThis)->m_bDragging = false;

	if(((xGUIWindow*)pThis)->m_bDragging)
	{
		pThis->SetX(pThis->GetX() + (g_xPaperAPI.m_xInput.m_lMouseX - lastmousex));
		pThis->SetY(pThis->GetY() + (g_xPaperAPI.m_xInput.m_lMouseY - lastmousey));
		lastmousex	= g_xPaperAPI.m_xInput.m_lMouseX;
		lastmousey	= g_xPaperAPI.m_xInput.m_lMouseY;

		xGUIWindowSyncRenderables((xGUIWindow*)pThis);
	}
}

void xGUIWindowClearState(xGUIObject* pThis, DWORD dwTime)
{
	((xGUIWindow*)pThis)->m_bDragging = false;

	xGUIObjectClearState(pThis, dwTime);
}

void xGUIWindowUpdate(xGUIObject* pThis, double dTime)
{
	if(((xGUIWindow*)pThis)->GetHidden())
		return;

	if(!pThis->m_xAnimation.m_bDone)
	{
		xAnimationUpdate(&pThis->m_xAnimation, dTime);
		xGUIWindowSyncRenderables((xGUIWindow*)pThis);
	}

	//((xGUIWindow*)pThis)->m_pWindow->Render(0, pThis->m_fX + ((xGUIWindow*)pThis)->m_fMainOffsetX, pThis->m_fY + ((xGUIWindow*)pThis)->m_fMainOffsetY, (int)pThis->m_fWidth - ((xGUIWindow*)pThis)->m_fTitleBarWidth, (int)pThis->m_fHeight - ((xGUIWindow*)pThis)->m_fTitleBarHeight, pThis->GetAbsoluteScaleX(), pThis->GetAbsoluteScaleY(), pThis->GetRotation(), pThis->GetAbsoluteColor());
	//if(((xGUIWindow*)pThis)->m_bTitleBar)
	{
		//((xGUIWindow*)pThis)->m_pTitleBar->Render(0, pThis->m_fX + ((xGUIWindow*)pThis)->m_fTitleBarX, pThis->m_fY + ((xGUIWindow*)pThis)->m_fTitleBarY);

		//if(((xGUIWindow*)pThis)->m_bCloseButton)
		//	((xGUIWindow*)pThis)->m_pCloseButton->Render(0, pThis->m_fX + ((xGUIWindow*)pThis)->m_fCloseButtonX, pThis->m_fY + ((xGUIWindow*)pThis)->m_fCloseButtonY);
	}	

	xGUIObjectUpdate(pThis, dTime);
}

void xGUIWindowUpdateRenderables(xGUIObject* pThis)
{
	xGUIWindowSyncRenderables((xGUIWindow*)pThis);
}

void xGUIWindowRefreshRenderables(xGUIObject* pThis)
{
	xGUIWindowSyncRenderables((xGUIWindow*)pThis, true);
}

void xGUIWindow::Shutdown()
{
	xGUIObject::Shutdown();

	if(m_pWindow)
		xSceneGraphDeleteQuad(m_pSceneGraph, m_pWindow, false);
}

void xGUIWindow::Hide()
{
	SetHidden(true);
	UpdateRenderables();
	m_bHasFocus = false;
}

void xGUIWindow::Show()
{
	SetHidden(false);
	UpdateRenderables();
	m_pGUI->SetFocus(m_strName);
}

int xGUIWindow::LoadWindow(float fDepth)
{
	xRenderable2DInit(&m_xRenderable);

	xGUIObject::LoadObject("");

	SetDepth(fDepth);

	m_bDragable			= m_pParser->ToBool("m_bDragable");
	m_bDragTitleOnly	= m_pParser->ToBool("m_bDragTitleOnly");
	m_bClosable			= m_pParser->ToBool("m_bClosable");
	m_bCollapsable		= m_pParser->ToBool("m_bCollapsable");
	m_bCollapsed		= m_pParser->ToBool("m_bCollapsed");
	m_bAlwaysOnTop		= m_pParser->ToBool("m_bAlwaysOnTop");
	m_bAlwaysOnBottom	= m_pParser->ToBool("m_bAlwaysOnBottom");
	m_bAlwaysFocused	= m_pParser->ToBool("m_bAlwaysFocused");
	m_bSizable			= m_pParser->ToBool("m_bSizable");
	m_strThemeName		= m_pParser->GetVariable("m_strThemeName");

	m_pFont = m_pGUI->GetFont(m_pParser->GetVariable("m_strFontName"));

	char tmp[256];
	sprintf(tmp, "Data/Themes/%s/Theme.pet", m_strThemeName);

	xParser themeparser;
	themeparser.OpenScript(tmp);

	char function[256], params[3][52];
	themeparser.ReadLine(function, params[0], params[1], params[2]);

	m_bTitleBar				= themeparser.ToBool("m_bTitleBar");
	m_bCloseButton			= themeparser.ToBool("m_bCloseButton");
	m_fTitleBarX			= themeparser.ToFlt ("m_fTitleBarX");
	m_fTitleBarY			= themeparser.ToFlt ("m_fTitleBarY");
	m_fCloseButtonX			= themeparser.ToFlt ("m_fCloseButtonX");
	m_fCloseButtonY			= themeparser.ToFlt ("m_fCloseButtonY");
	m_fCloseButtonHeight	= themeparser.ToFlt ("m_fCloseButtonHeight");
	m_fCloseButtonWidth		= themeparser.ToFlt ("m_fCloseButtonWidth");

	bool  titlevert			= themeparser.ToBool("m_bTitleBarVert");
	float titlethickness	= themeparser.ToFlt ("m_fTitleBarThickness");

	themeparser.EndScript();

	char *tempvariable = m_pParser->GetVariable("m_strPostLoadCommand");
	if(tempvariable && tempvariable[0] != '\0')
	{
		m_strPostLoadCommand = new char[256];
		memset(m_strPostLoadCommand, 0, 256);
		GetCommandAndParams(m_strPostLoadCommand, m_strPostLoadParams, tempvariable);
	}

	tempvariable = m_pParser->GetVariable("m_strKeyPressCommand");
	if(tempvariable && tempvariable[0] != '\0')
	{
		m_strKeyPressCommand = new char[256];
		memset(m_strKeyPressCommand, 0, 256);
		GetCommandAndParams(m_strKeyPressCommand, m_strKeyPressParams, tempvariable);
	}

	if(m_bCloseButton)
	{
		sprintf(tmp, "Data/Themes/%s/CloseButton.p2d", m_strThemeName);
		//m_pCloseButton = g_xPaperAPI.m_xGraphics.Create2DObject(tmp);
	}

	if(m_bTitleBar)
	{
		sprintf(tmp, "Data/Themes/%s/TitleBar.p2d", m_strThemeName);
		//m_pTitleBar = g_xPaperAPI.m_xGraphics.Create2DObject(tmp);

		if(!titlevert)
		{
			m_fTitleBarWidth  = m_fMainOffsetX = 0;
			m_fTitleBarHeight = m_fMainOffsetY = titlethickness;
			
			//m_pTitleBar->m_iWidth  = m_fWidth  - m_fTitleBarX;
		}
		else
		{
			m_fTitleBarWidth  = m_fMainOffsetX = titlethickness;
			m_fTitleBarHeight = m_fMainOffsetY = 0;
			
			//m_pTitleBar->m_iHeight = m_fHeight - m_fTitleBarY;
		}
	}
	else
		m_fTitleBarWidth  = m_fMainOffsetX = m_fTitleBarHeight = m_fMainOffsetY = 0;

	tempvariable = m_pParser->GetVariable("m_strBackground");
	if(tempvariable && tempvariable[0] != '\0')
		strcpy(tmp, tempvariable);
	else
		sprintf(tmp, "Data/Themes/%s/Window.p2d", m_strThemeName);

	if(m_bStretch)
	{
		m_pWindow = xSceneGraphCreateQuad(tmp, 0);
		xRenderable2DQuadSetSize(m_pWindow, GetWidth(), GetHeight());
		xSceneGraphInsertQuad(m_pSceneGraph, m_pWindow, 9, fDepth);
		xRenderable2DQuad9Update(m_pWindow);
	}
	else
	{
		m_pWindow = xSceneGraphCreateQuad(tmp, 0);
		xRenderable2DQuadSetSize(m_pWindow, GetWidth(), GetHeight());
		xSceneGraphInsertQuad(m_pSceneGraph, m_pWindow, 1, fDepth);
		xRenderable2DQuadUpdate(m_pWindow);
	}

	int selectiongroupcount = m_pParser->ToInt("m_iSelectionGroupCount");
	if(selectiongroupcount > 0)
	{
		m_pSelectionGroups = new xGUISelectionGroup*[selectiongroupcount];

		for(int i = 0; i < selectiongroupcount; i++)
			m_pSelectionGroups[i] = new xGUISelectionGroup;
	}

	return PE_RES_NOERROR;
}

void xGUIWindow::UpdateWindowDepth(float fNewDepth)
{
	SetDepth(fNewDepth);

	xRenderable2DQuadUpdateDepth(m_pWindow, m_pSceneGraph, GetAbsoluteDepth());

	xGUIObjectUpdateDepth(this);
}

FORCE_INLINE void xGUIWindowSyncRenderables(xGUIWindow *pWindow, bool bRefresh)
{
	xColor tempcolor;

	xRenderable2DCopyTransform(&pWindow->m_pWindow->m_xRenderable2D, &pWindow->m_xRenderable);
	xRenderable2DUpdateAlpha(&pWindow->m_pWindow->m_xRenderable2D, pWindow->GetAbsoluteColor(&tempcolor)->A);

	xRenderable2DQuadUpdateVisibility(pWindow->m_pWindow, !pWindow->GetAbsoluteHidden());

	if(bRefresh)
		xRenderable2DQuadUpdateTexture(pWindow->m_pWindow, pWindow->m_pWindow->m_pTextures[0]);

	if(pWindow->m_bStretch)
		xRenderable2DQuad9Update(pWindow->m_pWindow);
	else
		xRenderable2DQuadUpdate(pWindow->m_pWindow);

	xGUIObjectList *curobj = pWindow->m_xChildObjectList.m_pNext;
	while(curobj)
	{
		if(bRefresh && curobj->m_pData->pfnRefreshRenderables)
			curobj->m_pData->RefreshRenderables();

		curobj->m_pData->UpdateRenderables();
		curobj = curobj->m_pNext;
	}
}

xGUIObjectList* xGUIWindow::GetNextTabIndexObject(xGUIObjectList *pCurrentlyFocused)
{
	xGUIObjectList *newfocusobj = NULL;
	int focusindex;

	if(pCurrentlyFocused)
	{
		focusindex = pCurrentlyFocused->m_pData->m_iTabIndex + 1;
		for(newfocusobj = m_xChildObjectList.m_pNext; newfocusobj; newfocusobj = newfocusobj->m_pNext)
			if(newfocusobj->m_pData->m_iTabIndex == focusindex)
				break;

		if(!newfocusobj)
			for(newfocusobj = m_xChildObjectList.m_pNext; newfocusobj; newfocusobj = newfocusobj->m_pNext)
				if(newfocusobj->m_pData->m_iTabIndex == 0)
					break;
	}

	return newfocusobj;
}