/////////////////////////////////////////////////////////////////
// Name: xGame.h
/////////////////////////////////////////////////////////////////
// Description:
// - Serves as an object for each different type of game mode.
/////////////////////////////////////////////////////////////////
// - 7.15.2004, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
#ifndef XGAME_H_
#define XGAME_H_

#include "xGameDefines.h"
#include "xOptions.h"
#include "xGameOptions.h"
#include "xLocalPlayer.h"
#include "xNetPlayer.h"
#include "xSong.h"

enum xCrowdState
{
	BB_CROWD_NONE,
	BB_CROWD_NORMAL,
	BB_CROWD_EXCITED
};

enum xEnemyMoveSoundState
{
	BB_ENEMYMOVE_UNDECIDED = 0,
	BB_ENEMYMOVE_PREMOVE,
	BB_ENEMYMOVE_POSTMOVE,
	BB_ENEMYMOVE_POSTMOVEDONE
};

extern int		 g_iTourLevelCount[7];
extern int		 g_iTourEnemyIndices[7][12];
extern int		 g_iPlayerChargeClearableCount[5];
extern float	 g_fPlayerDamageMultipliers[BB_NUMDIFFICULTIES];
extern float	 g_fEnemyDamageMultipliers[BB_NUMDIFFICULTIES];
extern char		*g_strAchievements[BB_ACHIEVEMENTS_MAX];
extern char		*g_strAchievementTitle[BB_ACHIEVEMENTS_MAX];
extern char		*g_strChallengeTitle[BB_ACHIEVEMENTS_MAX];
extern char		*g_strChallengeDescription[BB_ACHIEVEMENTS_MAX][4];
extern int		 g_iChallengeDifficulties[BB_ACHIEVEMENTS_MAX];
extern char		*g_strChallengeSongList[BB_ACHIEVEMENTS_MAX];
extern int		 g_iChallengeSongDifficulties[BB_ACHIEVEMENTS_MAX];
extern char		*g_strAllEnemies[BB_ENEMIES_MAX];

extern xOptions g_xOptions;

void xGameRunPostLoadCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow);
void xGameRunInputCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xGameRunKeyCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xGameRunHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xGUIObject* &pOutHoverObject);
void xGameRunUnHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xGameRunDragCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xRenderable2DQuad* &pOutDragObject);
void xGameRunDropCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);

void xGameOnPlayerAnimationCompleted(xMD3Model *pModel, void *pCallbackObject = 0);
void xGameOnEnemyAnimationCompleted(xMD3Model *pModel, void *pCallbackObject = 0);

class xGame : public xGameState
{
public:
	xGame()
	{
		m_bFinishedConnecting			= 1;
		m_iConnectingStatus				= 0;
		m_pRemotePlayer					= 0;
		m_bDonePrecache					= 0;
		m_bPrePopup						= 0;
		m_bSenseiPrePopup				= 0;
		m_pSenseiPrePopupSound			= 0;
		m_pSenseiPrePopupVoice			= 0;
		m_bCountdown					= 0;
		m_bPaused						= 0;
		m_bFailed						= 0;
		m_bTypingMessage				= 0;
		m_iTypedMessageLength			= 0;
		m_pMessageFont					= 0;
		m_bVersusScreenAnimating		= 0;
		m_pVersusBackground				= 0;
		m_pVersusVSImage				= 0;
		m_pVersusPlayerPortrait			= 0;
		m_pVersusEnemyPortrait			= 0;
		m_pVersusScreenSound			= 0;
		m_pBackground					= 0;
		m_pPlayerModel					= 0;
		m_pEnemyModel					= 0;
		m_pCountdown					= 0;
		m_pCrowdBarArrow1				= 0;
		m_bFoundArrowStart				= 0;
		m_fCurArrowTime					= 0;
		m_dCurBeatTime					= 0.0;
		m_dInitTime						= 0.0;
		m_dwGameStartTime				= 0;
		m_dwOldGameTime					= 0;
		m_fCrowdBarPercentage			= 0.5f;
		m_pScoreCountLongSound			= 0;
		m_pCountdownSounds[0]			= 0;
		m_pEnemyIntroSound				= 0;
		m_pPlayerNameBG					= 0;
		m_pEnemyNameBG					= 0;
		m_bStopSounds					= 0;
		m_dwTimerGrowTime				= 0;
		m_dwNextTimerGrowTime			= 0;
		m_bColorizeTimer				= 0;
		m_pTutorialDimmer				= 0;
		m_bPlayedHitSound				= 0;
		m_iSecondaryPiecesInARow		= 0;
		m_pCrowdLoopSound				= 0;
		m_pTutorialVoiceSounds[0]		= 0;
		m_iTutVoiceIndex				= -1;
		m_iSenseiTutVoiceIndex			= -1;
		m_dwEnemyPostMoveSoundTime		= 2999999999;
		m_xEnemyMoveState				= BB_ENEMYMOVE_UNDECIDED;
		m_dwPlayerVoiceTime				= 0;
		m_pAchievementIcon[0]			= 0;

		m_xGUI.pfnRunPostLoadCommand	= &xGameRunPostLoadCommand;
		m_xGUI.pfnRunInputCommand		= &xGameRunInputCommand;
		m_xGUI.pfnRunKeyCommand			= &xGameRunKeyCommand;
		m_xGUI.pfnRunHoverCommand		= &xGameRunHoverCommand;
		m_xGUI.pfnRunUnHoverCommand		= &xGameRunUnHoverCommand;
		m_xGUI.pfnRunDragCommand		= &xGameRunDragCommand;
		m_xGUI.pfnRunDropCommand		= &xGameRunDropCommand;

		for(int i = 0; i < BB_TEXTMESSAGES_MAX; i++)
			m_strTextMessages[i] = 0;
	}

	void Init();
	bool InitListenServer(const char *strMyIP, int iPort, const char *strBackgroundPath, const char *strSongName, int iDifficultyLevel);
	bool InitClient(const char *strConnectToAddress, int iPort);
	void SetupGameOptionsForMultiplayer(const char *strEnemyName, const char *strBackgroundName, const char *strSongName, xGameDifficulty iDifficultyLevel);
	void ResetListenServerConnection();
	void DelayedInit(const char *strLevelName, float fDelay);

	void InitVersusScreen(const char *strLevelName);
	bool Init(const char *strLevelName);
	bool PreUpdate(DWORD dwTime);
	bool Update(double dTime);
	void RefreshAllRenderables();
	void Shutdown();

	void CopyLevelPath(const char *strLevelName, char *strLevelPath);
	void GetPreloadGameOptions(const char *strLevelName, char *strEnemyName = 0, char *strEnemyDisplayName = 0, char *strSongName = 0, char *strArtistName = 0, char *strNextLevelName = 0, xPreloadGameOptions *pDest = 0);
	void InitializeEnemyName();
	void InitTimeString(int iTime);

	             void  AdjustCrowdBar(float fShift);
	             float GetCrowdBarPosition() { return m_fCrowdBarPercentage; }
	FORCE_INLINE float GetCrowdBarBlocksClearedAdjustment(int iBlockCount);
	FORCE_INLINE float GetCrowdBarFailedToAddBlockAdjustment(xGameDifficulty iDifficulty);
	FORCE_INLINE float GetCrowdBarMissedBeatAdjustment(xGameDifficulty iDifficulty);
	FORCE_INLINE float GetCrowdBarMissedComboAdjustment(xGameDifficulty iDifficulty);

	void HandleNetworkPackets();
	void HandlePingPackets();

	void ParseLine(char *strLine);

	xGUIWindowList* AddWindow(const char *strWindowName);
	void			AddPrePopupWindow();
	void			AddSenseiPrePopupWindow();
	void			AddClientPrePopupWindow();
	void			AddServerPrePopupWindow();
	void			AddPostPopupWindow();
	void			UpdatePostPopup(double dTime);
	void			AddFailedPostPopupWindow();
	void			AddClientPostPopupWindow();
	void			AddServerPostPopupWindow();
	void			ReplacePrePopupText(char *strDestination, const char *strSource);
	void			UpdatePrePopupText();
	void			UpdateSenseiPrePopupText();
	void			SelectPlayerMoveCategory(xGUIWindowList *pWindow, const char* strCategory);

	void Ready();
	void SendReady();

	void PlayVoiceSound(xSound *pSound) { if(m_pVoiceSoundPlaying && m_pVoiceSoundPlaying->IsPlaying()) return; m_pVoiceSoundPlaying = pSound; pSound->Play(g_xOptions.m_fVoiceOverVolume); }

	void ShiftMessagesUp();

	void InitTutorial();
	void InitTutorialField();
	void UpdateTutorial();
	void ShowTutorialText(const char *strText, int iTutVoiceIndex = -1, bool bRequireEnterPress = 1, const char *strSenseiText = 0, int iSenseiTutVoiceIndex = -1, int iSenseiTutVoiceIndex2 = -1);
	void ShowRotateAndThrowKeys();
	void ShowKeys(bool bShowBottomKey, bool bShowTutorialDimmer = 1);
	void HideAllKeys(bool bDelete);
	void ResetFieldRotation();
	void ResetTutorialOnDropFailure(const char *strText, int iTutVoiceIndex = -1);

	void UnlockAchievement(int iAchievementID);

	FORCE_INLINE void Pause(bool bHasFocus);
	FORCE_INLINE void UnPause();

	void Failed();
	bool GameOver() { return m_bFailed; }

	xSceneGraph	m_xSceneGraph;
	xSong		m_xSong;
	xGUI		m_xGUI;
	DWORD		m_dwGameStartTime;
	DWORD		m_dwOldGameTime;
	double		m_dCurBeatTime;
	double		m_dInitTime;
	char		m_strStoredLevel[32];
	float		m_fBeatPercentage;
	float		m_fBeatPercentageLinear;
	DWORD		m_dwTimerGrowTime;
	DWORD		m_dwNextTimerGrowTime;
	bool		m_bColorizeTimer;

	// Network variables
	xHTTPQuery	 m_xHTTPQuery;
	bool		 m_bFinishedConnecting;
	int			 m_iConnectingStatus;
	bool		 m_bReady;
	bool		 m_bSentReady;
	bool		 m_bReceivedPing;
	DWORD		 m_dwServerStartTime;
	char		*m_strTextMessages[BB_TEXTMESSAGES_MAX];
	xFont		*m_pMessageFont;

	// Versus Screen
	bool				 m_bVersusScreenAnimating;
	float				 m_fVersusAnimationTime;
	DWORD				 m_dwVersusScreenStartTime;
	xRenderable2DQuad	*m_pVersusBackground;
	xRenderable2DQuad	*m_pVersusVSImage;
	xRenderable2DQuad	*m_pVersusPlayerPortrait;
	xRenderable2DQuad	*m_pVersusEnemyPortrait;
	xRenderable2DText	 m_xVersusPlayerName;
	xRenderable2DText	 m_xVersusEnemyName;
	xRenderable2DText	 m_xVersusSongName;
	xRenderable2DText	 m_xVersusArtistName;
	xSound				*m_pVersusScreenSound;
	bool				 m_bPlayedVersusScreenSound;

	// Main Renderables
	xRenderable2DQuad		 *m_pBackground;
	xMD3Model				 *m_pEnemyModel;
	xMD3Model				 *m_pPlayerModel;
	xRenderable2DQuad		 *m_pCountdown;
	xTexture				**m_pCountdownTextures;
	xRenderable2DQuad		 *m_pCrowdBarArrow1;
	xRenderable2DQuad		 *m_pCrowdBarArrow2;
	float					  m_fCrowdBarWidth;
	float					  m_fCrowdBarHeight;
	float					  m_fCrowdBarPercentage; //0.0 is full Player favored, 1.0 is fully Enemy, starts at 0.5
	float					  m_fCrowdBarDisplayedPercentage;
	float					  m_fCrowdBarMomentum;
	xRenderable2DQuad		 *m_pTimerBG;
	xRenderable2DText		  m_xTimer;
	xRenderable2DQuad		 *m_pPlayerChargeBG;
	xRenderable2DQuad		 *m_pPlayerCharges[4];
	xRenderable2DQuad		 *m_pPlayerChargeOverlays[4];
	DWORD					  m_dwPlayerMoveFlashTime;
	xRenderable2DQuad		 *m_pPlayerNameBG;
	xRenderable2DText		  m_xPlayerName;
	xRenderable2DText		  m_xPlayerRank;
	xRenderable2DQuad		 *m_pEnemyChargeBG;
	xRenderable2DQuad		 *m_pEnemyCharge;
	xRenderable2DQuad		 *m_pEnemyChargeOverlay;
	xRenderable2DQuad		 *m_pEnemyNameBG;
	xRenderable2DText		  m_xEnemyName;
	xRenderable2DText		  m_xEnemyRank;
	char					  m_strPlayerRank[10];
	char					  m_strRemotePlayerRank[10];
	int						  m_iEnemyRankWidth;
	DWORD					  m_dwLastEnemyMoveTime;
	DWORD					  m_dwNextEnemyMoveTime;
	DWORD					  m_dwUsedEnemy3MoveTime;
	DWORD					  m_dwEnemyMoveFlashTime;
	int						  m_iEnemyMovesExecuted;
	DWORD					  m_dwNextBossInteraction;
	float					  m_fEnemyDamage;

	// PrePopup Status
	bool		 m_bPrePopup;
	int			 m_iPrePopupTextIndex;
	bool		 m_bPrePopupMoreLines;
	float		 m_fPrePopupMoreLinesTime;
	xGUIText	*m_pPrePopupMoreLinesText;
	bool		 m_bSenseiPrePopup;
	int			 m_iSenseiPrePopupTextIndex;
	bool		 m_bSenseiPrePopupMoreLines;
	float		 m_fSenseiPrePopupMoreLinesTime;
	xGUIText	*m_pSenseiPrePopupMoreLinesText;
	xGUIText	*m_pPrePopupPressEnterText;
	double		 m_dPrePopupPressEnterTime;
	xSound		*m_pSenseiPrePopupSound;
	bool		 m_bPlayedSenseiPrePopupSound;
	xSound		*m_pSenseiPrePopupVoice;
	bool		 m_bRequireEnterPrePopup;

	// PostPopup Status
	xGUIWindow		*m_pPostPopupWindow;
	xGUIText		*m_pScoreText;
	xGUIImage		*m_pDominatedImage;
	xGUIImage		*m_pNewRecordImage;
	xSound			*m_pScoreCountLongSound;
	xSound			*m_pUnlockedSound;
	int				 m_iPostPopupStep;
	double			 m_dPostPopupStepTime;
	bool			 m_bPostPopupDominationBonus;
	int				 m_iPostPopupOldHighScore;
	DWORD			 m_dwPostPopupCrowdScore;
	bool			 m_bPostPopupNewHighScore;
	float			 m_fStartingScoreTextY;
	int				 m_iPostPopupAwardsShown;
	int				 m_iPostPopupAwardsUnlocked;
	int				 m_iPostPopupPrevAwardScore;
	int				 m_iPostPopupNextAwardScore;

	// Tutorial Helpers
	int					 m_iTutorialStep;
	bool				 m_bPlayedHitSound;
	int					 m_iBoardRotateKeyPressed;
	bool				 m_bAllowFieldRotation;
	bool				 m_bPieceRotateKeyPressed;
	bool				 m_bDropKeyPressed;
	bool				 m_bPieceDropped;
	DWORD				 m_dwTutorialTime;
	DWORD				 m_dwTutorialPeteStartTime;
	xRenderable2DQuad	*m_pTutorialDimmer;
	xRenderable2DQuad	*m_pTopKey;
	xRenderable2DText	 m_xTopKeyText;
	xRenderable2DQuad	*m_pLeftKey;
	xRenderable2DText	 m_xLeftKeyText;
	xRenderable2DQuad	*m_pBottomKey;
	xRenderable2DText	 m_xBottomKeyText;
	xRenderable2DQuad	*m_pRightKey;
	xRenderable2DText	 m_xRightKeyText;
	xRenderable2DQuad	*m_pSoloKey;
	xRenderable2DText	 m_xSoloKeyText;
	xTexture			*m_pLeftArrowTexture;
	xTexture			*m_pRightArrowTexture;
	xTexture			*m_pDownArrowTexture;
	xTexture			*m_pUpArrowTexture;
	xTexture			*m_pBlankKeyTexture;

	// Countdown variables
	bool	 m_bCountdown;
	double	 m_dCountdownTime;
	float	 m_fOldCountdownPercent;
	xSound	*m_pCountdownSounds[3];
	double	 m_dEnemyIntroTime;

	// Crowd Sounds
	xCrowdState	 m_xCrowdState;
	xSound		*m_pCrowdLoopSound;
	xSound		*m_pCrowdLoopExcitedSound;
	xSound		*m_pCrowdPositiveSound;
	xSound		*m_pCrowdNegativeSound;
	xSound		*m_pCrowdGameWonSound;
	xSound		*m_pCrowdGameLostSound;

	// Enemy Sounds
	xSound					*m_pEnemyIntroSound;
	xSound					*m_pEnemyMoveSound;
	xSound					*m_pTutorialVoiceSounds[BB_TUTVOICE_MAX];
	int						 m_iTutVoiceIndex;
	int						 m_iSenseiTutVoiceIndex;
	xSound					*m_pEnemyIntroVoiceSound;
	xSound					*m_pEnemyPreMoveSound;
	xSound					*m_pEnemyPostMoveSound;
	xSound					*m_pEnemyReactionSound;
	xSound					*m_pWonVoiceSound;
	xSound					*m_pFailureVoiceSound;
	DWORD					 m_dwEnemyPostMoveSoundTime;
	xEnemyMoveSoundState	 m_xEnemyMoveState;
	DWORD					 m_dwPlayerVoiceTime;
	xSound					*m_pVoiceSoundPlaying;

	// Achievements
	int					  m_iAchievementStatus[BB_ACHIEVEMENTS_MAX];
	char				 *m_strAchievementsPending[BB_PENDINGACHIEVEMENTS_MAX];
	int					  m_iAchievementsUnlocking[BB_PENDINGACHIEVEMENTS_MAX];
	DWORD				  m_dwAchievementUnlockStart[BB_PENDINGACHIEVEMENTS_MAX];
	xRenderable2DQuad	 *m_pAchievementIcon[BB_PENDINGACHIEVEMENTS_MAX];
	xRenderable2DText	  m_xAchievementTitle[BB_PENDINGACHIEVEMENTS_MAX];
	xTexture			**m_pAchievementIconTextures;

	// Status variables
	bool   m_bDonePrecache;
	int    m_iGameSyncFrameCount;
	bool   m_bPaused;
	bool   m_bFailed;
	DWORD  m_dwCurrentTime;
	char   m_strTime[5];
	bool   m_bFoundArrowStart;
	float  m_fCurArrowTime;
	bool   m_bTypingMessage;
	int    m_iMessagePrefixLength;
	int    m_iTypedMessageLength;
	char   m_strMessageBuffer[BB_PACKET_MESSAGESIZE + 30];
	bool   m_bStopSounds;
	int    m_iSecondaryPiecesInARow;

	// Misc variables
	xLocalPlayer	 m_xPlayer;
	xNetPlayer		*m_pRemotePlayer;
	xGameOptions	*m_pParsedGameOptions;
};

void xGame::Pause(bool bHasFocus)
{
	m_bPaused = true;
	m_xSong.Stop();

	DEBUGLOG("Pause called\n");

	if(!bHasFocus || (!m_xPlayer.m_bWonGame && !g_xGame.m_bFailed))
	{
		if(m_xCrowdState == BB_CROWD_NORMAL)
		{
			DEBUGLOG("Pausing Normal Crowd Loop %i\n", m_pCrowdLoopSound->IsPlaying());
			if(m_pCrowdLoopSound->IsPlaying())
				m_pCrowdLoopSound->Pause();
		}
		else if(m_xCrowdState == BB_CROWD_EXCITED)
		{
			DEBUGLOG("Pausing Excited Crowd Loop %i\n", m_pCrowdLoopExcitedSound->IsPlaying());
			if(m_pCrowdLoopExcitedSound->IsPlaying())
				m_pCrowdLoopExcitedSound->Pause();
		}

		if(m_pVoiceSoundPlaying && m_pVoiceSoundPlaying->IsPlaying())
			m_pVoiceSoundPlaying->Pause();
	}
}

void xGame::UnPause()
{
	DEBUGLOG("UnPause called\n");
	if(!m_bSenseiPrePopup && (!m_bPrePopup || (g_xGameOptions.m_bTutorial && !m_bRequireEnterPrePopup)))
	{
		m_bPaused = false;

		if(!m_bCountdown)
		{
			m_xSong.Play(0, g_xOptions.m_fMusicVolume);

			m_dwGameStartTime = g_xPaperAPI.m_xPlatform.GetTime() - m_xSong.GetTime();
			m_iGameSyncFrameCount = 0;
		}
	}

	if(m_xCrowdState == BB_CROWD_NORMAL)
	{
		if(!m_pCrowdLoopSound->IsPlaying())
		{
			if(m_pCrowdLoopSound->IsPaused())
				m_pCrowdLoopSound->UnPause();
			else
				m_pCrowdLoopSound->Play(g_xOptions.m_fSoundVolume);
		}
	}
	else if(m_xCrowdState == BB_CROWD_EXCITED)
	{
		if(!m_pCrowdLoopExcitedSound->IsPlaying())
		{
			if(m_pCrowdLoopExcitedSound->IsPaused())
				m_pCrowdLoopExcitedSound->UnPause();
			else
				m_pCrowdLoopExcitedSound->Play(g_xOptions.m_fSoundVolume);
		}
	}

	if(m_pVoiceSoundPlaying && m_pVoiceSoundPlaying->IsPaused())
		m_pVoiceSoundPlaying->UnPause();
}

float xGame::GetCrowdBarBlocksClearedAdjustment(int iBlockCount)
{
	return pow((float)iBlockCount, 1.5f) / 15.811389f * BB_CROWDSHIFT_BLOCKSCLEARED * g_fPlayerDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
}

float xGame::GetCrowdBarFailedToAddBlockAdjustment(xGameDifficulty iDifficulty)
{
	switch(iDifficulty)
	{
		case BB_DIFFICULTY_TUTORIAL:
		case BB_DIFFICULTY_NOVICE:
			return BB_CROWDSHIFT_FAILEDTOADDBLOCK_BEGINNER * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_ADVANCED:
			return BB_CROWDSHIFT_FAILEDTOADDBLOCK_NOVICE * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_EXPERT:
			return BB_CROWDSHIFT_FAILEDTOADDBLOCK_EXPERT * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
	}
}

float xGame::GetCrowdBarMissedBeatAdjustment(xGameDifficulty iDifficulty)
{
	switch(iDifficulty)
	{
		case BB_DIFFICULTY_TUTORIAL:
		case BB_DIFFICULTY_NOVICE:
			return BB_CROWDSHIFT_MISSEDBEAT_BEGINNER * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_ADVANCED:
			return BB_CROWDSHIFT_MISSEDBEAT_NOVICE * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_EXPERT:
			return BB_CROWDSHIFT_MISSEDBEAT_EXPERT * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
	}
}

float xGame::GetCrowdBarMissedComboAdjustment(xGameDifficulty iDifficulty)
{
	switch(iDifficulty)
	{
		case BB_DIFFICULTY_TUTORIAL:
		case BB_DIFFICULTY_NOVICE:
			return BB_CROWDSHIFT_MISSEDCOMBO_BEGINNER * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_ADVANCED:
			return BB_CROWDSHIFT_MISSEDCOMBO_NOVICE * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
		case BB_DIFFICULTY_EXPERT:
			return BB_CROWDSHIFT_MISSEDCOMBO_EXPERT * g_fEnemyDamageMultipliers[g_xCurrentProfile.m_byteSelectedDifficulty];
	}
}

#endif