#include "peGame.h"

char xGameNetwork::m_strBuffer[MAX_NETWORK_BUFFERSIZE + 1] = {0};
int  xGameNetwork::m_iBufferOffset = 0;

void xGameNetwork::SendClientHandshake(const char* strName, int iRank)
{
	xBBNetPacketClientHandshake packet;

	CREATELOG();

	packet.m_ucPacketType = BB_NETPACKETTYPE_CLIENTHANDSHAKE;

	packet.m_ucNameSize = strlen(strName);

	if(packet.m_ucNameSize >= BB_PACKET_NAMESIZE)
		packet.m_ucNameSize = BB_PACKET_NAMESIZE - 1;

	memset(packet.m_strName, 0, BB_PACKET_NAMESIZE);
	strcpy_s(packet.m_strName, BB_PACKET_NAMESIZE, strName);

	*((int*)&packet.m_strName[packet.m_ucNameSize]) = iRank;

	packet.m_strName[packet.m_ucNameSize + 4] = '\0';

	LOG("NETWORK: SendClientHandshake - %i\n", packet.m_ucNameSize + 4);

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, packet.m_ucNameSize + 4);
}

void xGameNetwork::SendServerHandshake(const char* strName, const char* strBackground, const char* strSong, int iBeatLevel, int iRank)
{
	xBBNetPacketServerHandshake packet;

	CREATELOG();

	packet.m_ucPacketType = BB_NETPACKETTYPE_SERVERHANDSHAKE;

	packet.m_ucNameSize = strlen(strName);

	if(packet.m_ucNameSize >= BB_PACKET_NAMESIZE)
		packet.m_ucNameSize = BB_PACKET_NAMESIZE - 1;

	memset(packet.m_strName, 0, BB_PACKET_NAMESIZE);
	strcpy_s(packet.m_strName, BB_PACKET_NAMESIZE, strName);

	uint8_t offset = packet.m_ucNameSize;

	uint8_t size = strlen(strBackground);

	if(size >= BB_PACKET_BGSIZE)
		size = BB_PACKET_BGSIZE - 1;

	packet.m_strName[offset++] = size;

	memset(&packet.m_strName[offset], 0, BB_PACKET_BGSIZE);
	strcpy_s(&packet.m_strName[offset], BB_PACKET_BGSIZE, strBackground);

	offset += size;

	size = (uint8_t)strlen(strSong);

	if(size >= BB_PACKET_SONGSIZE)
		size = BB_PACKET_SONGSIZE - 1;

	packet.m_strName[offset++] = size;

	memset(&packet.m_strName[offset], 0, BB_PACKET_SONGSIZE);
	strcpy_s(&packet.m_strName[offset], BB_PACKET_SONGSIZE, strSong);

	offset += size;

	packet.m_strName[offset] = (uint8_t)iBeatLevel;
	*((int*)(packet.m_strName + offset + 1)) = iRank;

	packet.m_strName[offset + 5] = '\0';

	LOG("NETWORK: SendServerHandshake - %i\n", offset + 7);

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, offset + 7);
}

void xGameNetwork::SendSecurity(DWORD dwTime, const char* strExeMD5, const char* strBeatsMD5)
{
	xBBNetPacketSecurity packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_SECURITY;

	memset(packet.m_strEncryptedMD5s, 0, BB_PACKET_MD5SIZE);
	strcpy_s(packet.m_strEncryptedMD5s, 33, strExeMD5);
	strcpy_s(&packet.m_strEncryptedMD5s[32], 33, strBeatsMD5);

	LOG("NETWORK: SendSecurity - %i\n", sizeof(xBBNetPacketSecurity));

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketSecurity));
}

void xGameNetwork::SendAppearance()
{
	xBBNetPacketAppearance packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_APPEARANCE;

	LOG("NETWORK: SendAppearance - %i\n", sizeof(xBBNetPacketAppearance));

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketAppearance));
}

void xGameNetwork::SendReady()
{
	xBBNetPacketReady packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_READY;

	LOG("NETWORK: SendReady - %i\n", sizeof(xBBNetPacketReady));

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketReady));
}

void xGameNetwork::SendAddedBlock(int iComboCount, float fNewBarPosition)
{
	xBBNetPacketAddedBlock packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_ADDEDBLOCK;
	packet.m_i16ComboCount = iComboCount;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketAddedBlock));
}

void xGameNetwork::SendFinishedPiece(int iBlocksInPiece, float fNewBarPosition)
{
	xBBNetPacketFinishedPiece packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_FINISHEDPIECE;
	packet.m_ucBlocksInPiece = iBlocksInPiece;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketFinishedPiece));
}

void xGameNetwork::SendPushedDownPiece(int iDropDistance, float fNewBarPosition)
{
	xBBNetPacketPushedDownPiece packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_PUSHEDDOWNPIECE;
	packet.m_ucDropDistance = iDropDistance;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketPushedDownPiece));
}

void xGameNetwork::SendDroppedPiece(int iDropDistance, float fNewBarPosition)
{
	xBBNetPacketDroppedPiece packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_DROPPEDPIECE;
	packet.m_ucDropDistance = iDropDistance;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketDroppedPiece));
}

void xGameNetwork::SendBlocksCleared(int iBlockCount, float fNewBarPosition)
{
	xBBNetPacketBlocksCleared packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_BLOCKSCLEARED;
	packet.m_ucBlockCount = iBlockCount;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketBlocksCleared));
}

void xGameNetwork::SendFailedToAddBlock(float fNewBarPosition)
{
	xBBNetPacketFailedToAddBlock packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_FAILEDTOADDBLOCK;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketFailedToAddBlock));
}

void xGameNetwork::SendMissedBeat(int iComboCount, float fNewBarPosition)
{
	xBBNetPacketMissedBeat packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_MISSEDBEAT;
	packet.m_i16ComboCount = iComboCount;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketMissedBeat));
}

void xGameNetwork::SendMissedCombo(int iComboCount, float fNewBarPosition)
{
	xBBNetPacketMissedCombo packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_MISSEDCOMBO;
	packet.m_i16ComboCount = iComboCount;
	packet.m_fNewBarPosition = fNewBarPosition;

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketMissedCombo));
}

void xGameNetwork::SendRematch()
{
	xBBNetPacketRematch packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_REMATCH;

	LOG("NETWORK: SendRematch - %i\n", sizeof(xBBNetPacketRematch));

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketRematch));
}

void xGameNetwork::SendNextSong(char* strNextSong, int iBeatLevel)
{
	xBBNetPacketNextSong packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_NEXTSONG;

	packet.m_ucBeatLevel = iBeatLevel;

	packet.m_ucNextSongSize = strlen(strNextSong);

	if(packet.m_ucNextSongSize >= BB_PACKET_SONGSIZE)
		packet.m_ucNextSongSize = BB_PACKET_SONGSIZE - 1;

	memset(packet.m_strNextSong, 0, BB_PACKET_SONGSIZE);
	strcpy_s(packet.m_strNextSong, BB_PACKET_SONGSIZE, strNextSong);

	packet.m_strNextSong[packet.m_ucNextSongSize] = '\0';

	LOG("NETWORK: SendNextSong - %i\n", packet.m_ucNextSongSize + 3);

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, packet.m_ucNextSongSize + 3);
}

void xGameNetwork::SendAccept()
{
	xBBNetPacketAccept packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_ACCEPT;

	LOG("NETWORK: SendAccept - %i\n", sizeof(xBBNetPacketAccept));

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, sizeof(xBBNetPacketAccept));
}

void xGameNetwork::SendTextMessage(char *strMessage)
{
	xBBNetPacketTextMessage packet;

	packet.m_ucPacketType = BB_NETPACKETTYPE_MESSAGE;

	packet.m_ucTextMessageSize = strlen(strMessage);

	if(packet.m_ucTextMessageSize >= BB_PACKET_MESSAGESIZE)
		packet.m_ucTextMessageSize = BB_PACKET_MESSAGESIZE - 1;

	memset(packet.m_strTextMessage, 0, BB_PACKET_MESSAGESIZE);
	strcpy_s(packet.m_strTextMessage, BB_PACKET_MESSAGESIZE, strMessage);

	packet.m_strTextMessage[packet.m_ucTextMessageSize] = '\0';

	LOG("NETWORK: SendTextMessage - %i\n", packet.m_ucTextMessageSize + 2);

	g_xPaperAPI.m_xNetwork.Send((char*)&packet, packet.m_ucTextMessageSize + 2);
}

bool xGameNetwork::ReceivedNetPacket(int &iNetPacketType)
{
	// TODO: May need some way to handle incomplete packets, as this currently relies on NULL terminated complete packets
	//       Basically, if we encounter an incomplete packet, we copy the current buffer off to a backup buffer and look for the rest of it

	CREATELOG();

	LOG("NETWORK: PreReceivedNetPacket - %i - %i - %i - %i\n", m_iBufferOffset, g_xPaperAPI.m_xNetwork.m_iReadBufferOffset, (int)strlen(m_strBuffer), (int)strlen(&m_strBuffer[m_iBufferOffset]));

	if(m_strBuffer[m_iBufferOffset] == '\0')
	{
		m_iBufferOffset = 0;

		DWORD useless;
		g_xPaperAPI.m_xNetwork.Receive(m_strBuffer, useless);

		if(m_strBuffer[m_iBufferOffset] == '\0')
			return false;
	}

	iNetPacketType = (uint8_t)m_strBuffer[m_iBufferOffset];

	LOG("NETWORK: Received Network Packet - %i - %i\n", iNetPacketType, g_xGame.m_iConnectingStatus);

	switch(iNetPacketType)
	{
		case BB_NETPACKETTYPE_CLIENTHANDSHAKE:
		{
			if(*((int*)&m_strBuffer[m_iBufferOffset + 2 + (uint8_t)m_strBuffer[m_iBufferOffset + 1]]) == 0)
				return false;
		
			return true;
		}

		case BB_NETPACKETTYPE_SERVERHANDSHAKE:
		{
			int offset = m_iBufferOffset + 2 + (uint8_t)m_strBuffer[m_iBufferOffset + 1];
			offset += 1 + (uint8_t)m_strBuffer[offset];
			offset += 1 + (uint8_t)m_strBuffer[offset];
			if(*((int*)&m_strBuffer[offset]) == 0)
				return false;
		
			return true;
		}

		case BB_NETPACKETTYPE_SECURITY:
		{
			// TODO: Switch this to check the encrypted MD5s size
			if(m_strBuffer[63] == 0)
				return false;
		
			return true;
		}

		case BB_NETPACKETTYPE_APPEARANCE:
		{
			// TODO: Check the appearance packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_READY:
		{
			return true;
		}

		case BB_NETPACKETTYPE_ADDEDBLOCK:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_FINISHEDPIECE:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_PUSHEDDOWNPIECE:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_DROPPEDPIECE:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_BLOCKSCLEARED:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_FAILEDTOADDBLOCK:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_MISSEDBEAT:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_MISSEDCOMBO:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_REMATCH:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_NEXTSONG:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_ACCEPT:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}

		case BB_NETPACKETTYPE_MESSAGE:
		{
			// TODO: Check the packet to make sure it's complete
			return true;
		}
	}

	return false;
}

void xGameNetwork::ReceiveClientHandshake(char strName[BB_PACKET_SONGSIZE], int &iRank)
{
	// Skip over the Packet type
	m_iBufferOffset++;

	uint8_t namesize = m_strBuffer[m_iBufferOffset];
	if(namesize >= BB_PACKET_NAMESIZE)
		namesize = BB_PACKET_NAMESIZE - 1;
	m_iBufferOffset++;

	memset(strName, 0, BB_PACKET_NAMESIZE);
	for(int i = 0; i < namesize; i++)
		strName[i] = m_strBuffer[m_iBufferOffset + i];
	strName[namesize] = '\0';
	m_iBufferOffset += namesize;

	iRank = *((int*)(m_strBuffer + m_iBufferOffset));
	m_iBufferOffset += 4;

	LOG("NETWORK: ReceiveClientHandshake - %i\n", m_iBufferOffset);
}

void xGameNetwork::ReceiveServerHandshake(char strName[BB_PACKET_SONGSIZE], char strBackground[BB_PACKET_SONGSIZE], char strSong[BB_PACKET_SONGSIZE], int &iBeatLevel, int &iRank)
{
	// Skip over the Packet type
	m_iBufferOffset++;

	uint8_t size = m_strBuffer[m_iBufferOffset];
	if(size >= BB_PACKET_NAMESIZE)
		size = BB_PACKET_NAMESIZE - 1;
	m_iBufferOffset++;

	memset(strName, 0, BB_PACKET_NAMESIZE);
	for(int i = 0; i < size; i++)
		strName[i] = m_strBuffer[m_iBufferOffset + i];
	strName[size] = '\0';
	m_iBufferOffset += size;

	size = m_strBuffer[m_iBufferOffset];
	if(size >= BB_PACKET_BGSIZE)
		size = BB_PACKET_BGSIZE - 1;
	m_iBufferOffset++;

	memset(strBackground, 0, BB_PACKET_BGSIZE);
	for(int i = 0; i < size; i++)
		strBackground[i] = m_strBuffer[m_iBufferOffset + i];
	strBackground[size] = '\0';
	m_iBufferOffset += size;

	size = m_strBuffer[m_iBufferOffset];
	if(size >= BB_PACKET_SONGSIZE)
		size = BB_PACKET_SONGSIZE - 1;
	m_iBufferOffset++;

	memset(strSong, 0, BB_PACKET_SONGSIZE);
	for(int i = 0; i < size; i++)
		strSong[i] = m_strBuffer[m_iBufferOffset + i];
	strSong[size] = '\0';
	m_iBufferOffset += size;

	iBeatLevel = (uint8_t)m_strBuffer[m_iBufferOffset];
	m_iBufferOffset++;

	iRank = *((int*)(m_strBuffer + m_iBufferOffset));
	m_iBufferOffset += 4;

	memcpy(g_xGameOptions.m_strEnemyDisplayName, strName, BB_PACKET_SONGSIZE);
	LOG("NETWORK: ReceiveServerHandshake - %i\n", m_iBufferOffset);
}

void xGameNetwork::ReceiveSecurity(char strExeMD5[33], char strBeatsMD5[33])
{
	xBBNetPacketSecurity* packet = (xBBNetPacketSecurity*)&m_strBuffer[m_iBufferOffset];

	memset(strExeMD5, 0, 33);
	memcpy(strExeMD5, packet->m_strEncryptedMD5s, 32);

	memset(strBeatsMD5, 0, 33);
	memcpy(strBeatsMD5, &packet->m_strEncryptedMD5s[32], 32);

	m_iBufferOffset += sizeof(xBBNetPacketSecurity);

	LOG("NETWORK: ReceiveSecurity - %i\n", sizeof(xBBNetPacketSecurity));
}

void xGameNetwork::ReceiveAppearance()
{
	m_iBufferOffset += sizeof(xBBNetPacketAppearance);

	LOG("NETWORK: ReceiveAppearance - %i\n", sizeof(xBBNetPacketAppearance));
}

void xGameNetwork::ReceiveReady()
{
	m_iBufferOffset += sizeof(xBBNetPacketReady);

	LOG("NETWORK: ReceiveReady - %i\n", sizeof(xBBNetPacketReady));
}

void xGameNetwork::ReceiveAddedBlock(int &iComboCount, float &fNewBarPosition)
{
	xBBNetPacketAddedBlock* packet = (xBBNetPacketAddedBlock*)&m_strBuffer[m_iBufferOffset];

	iComboCount = packet->m_i16ComboCount;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketAddedBlock);
}

void xGameNetwork::ReceiveFinishedPiece(int &iBlocksInPiece, float &fNewBarPosition)
{
	xBBNetPacketFinishedPiece* packet = (xBBNetPacketFinishedPiece*)&m_strBuffer[m_iBufferOffset];

	iBlocksInPiece = packet->m_ucBlocksInPiece;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketFinishedPiece);
}

void xGameNetwork::ReceivePushedDownPiece(int &iDropDistance, float &fNewBarPosition)
{
	xBBNetPacketPushedDownPiece* packet = (xBBNetPacketPushedDownPiece*)&m_strBuffer[m_iBufferOffset];

	iDropDistance = packet->m_ucDropDistance;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketPushedDownPiece);
}

void xGameNetwork::ReceiveDroppedPiece(int &iDropDistance, float &fNewBarPosition)
{
	xBBNetPacketDroppedPiece* packet = (xBBNetPacketDroppedPiece*)&m_strBuffer[m_iBufferOffset];

	iDropDistance = packet->m_ucDropDistance;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketDroppedPiece);
}

void xGameNetwork::ReceiveBlocksCleared(int &iBlockCount, float &fNewBarPosition)
{
	xBBNetPacketBlocksCleared* packet = (xBBNetPacketBlocksCleared*)&m_strBuffer[m_iBufferOffset];

	iBlockCount = packet->m_ucBlockCount;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketBlocksCleared);
}

void xGameNetwork::ReceiveFailedToAddBlock(float &fNewBarPosition)
{
	xBBNetPacketFailedToAddBlock* packet = (xBBNetPacketFailedToAddBlock*)&m_strBuffer[m_iBufferOffset];

	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketFailedToAddBlock);
}

void xGameNetwork::ReceiveMissedBeat(int &iComboCount, float &fNewBarPosition)
{
	xBBNetPacketMissedBeat* packet = (xBBNetPacketMissedBeat*)&m_strBuffer[m_iBufferOffset];

	iComboCount = packet->m_i16ComboCount;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketMissedBeat);
}

void xGameNetwork::ReceiveMissedCombo(int &iComboCount, float &fNewBarPosition)
{
	xBBNetPacketMissedCombo* packet = (xBBNetPacketMissedCombo*)&m_strBuffer[m_iBufferOffset];

	iComboCount = packet->m_i16ComboCount;
	fNewBarPosition = packet->m_fNewBarPosition;

	m_iBufferOffset += sizeof(xBBNetPacketMissedCombo);
}

void xGameNetwork::ReceiveRematch()
{
	m_iBufferOffset += sizeof(xBBNetPacketRematch);

	LOG("NETWORK: ReceiveRematch - %i\n", sizeof(xBBNetPacketRematch));
}

void xGameNetwork::ReceiveNextSong(char strNextSong[BB_PACKET_SONGSIZE], xGameDifficulty &iBeatLevel)
{
	// Skip over the Packet type
	m_iBufferOffset++;

	iBeatLevel = (xGameDifficulty)(uint8_t)m_strBuffer[m_iBufferOffset];
	m_iBufferOffset++;

	uint8_t songsize = m_strBuffer[m_iBufferOffset];
	if(songsize >= BB_PACKET_SONGSIZE)
		songsize = BB_PACKET_SONGSIZE - 1;
	m_iBufferOffset++;

	memset(strNextSong, 0, BB_PACKET_SONGSIZE);
	for(int i = 0; i < songsize; i++)
		strNextSong[i] = m_strBuffer[m_iBufferOffset + i];
	strNextSong[songsize] = '\0';
	m_iBufferOffset += songsize;

	LOG("NETWORK: ReceiveNextSong - %i\n", m_iBufferOffset);
}

void xGameNetwork::ReceiveAccept()
{
	m_iBufferOffset += sizeof(xBBNetPacketAccept);

	LOG("NETWORK: ReceiveAccept - %i\n", sizeof(xBBNetPacketAccept));
}

void xGameNetwork::ReceiveTextMessage(char strMessage[BB_PACKET_MESSAGESIZE])
{
	// Skip over the Packet type
	m_iBufferOffset++;

	uint8_t messagesize = m_strBuffer[m_iBufferOffset];
	if(messagesize >= BB_PACKET_MESSAGESIZE)
		messagesize = BB_PACKET_MESSAGESIZE - 1;
	m_iBufferOffset++;

	memset(strMessage, 0, BB_PACKET_MESSAGESIZE);
	for(int i = 0; i < messagesize; i++)
		strMessage[i] = m_strBuffer[m_iBufferOffset + i];
	strMessage[messagesize] = '\0';
	m_iBufferOffset += messagesize;

	LOG("NETWORK: ReceiveTextMessage - %i\n", m_iBufferOffset);
}
