#include "Paper.h"

xGraphics g_xGraphics;


#ifndef __FRAGSHADER_NOARB
PFNGLGENPROGRAMSARBPROC				glGenProgramsARB = 0;
PFNGLBINDPROGRAMARBPROC				glBindProgramARB = 0;
PFNGLPROGRAMSTRINGARBPROC			glProgramStringARB = 0;
PFNGLACTIVETEXTUREARBPROC			glActiveTextureARB = 0;
PFNGLMULTITEXCOORD2FARBPROC			glMultiTexCoord2fARB = 0;
PFNGLPROGRAMLOCALPARAMETER4FARBPROC	glProgramLocalParameter4fARB = 0;
#else // __FRAGSHADER_NOARB
PFNGLCREATESHADERPROC				glCreateShader = 0;
PFNGLSHADERSOURCEPROC				glShaderSource = 0;
PFNGLCOMPILESHADERPROC				glCompileShader = 0;
PFNGLGETSHADERINFOLOGPROC			glGetShaderInfoLog = 0;
PFNGLCREATEPROGRAMPROC				glCreateProgram = 0;
PFNGLATTACHSHADERPROC				glAttachShader = 0;
PFNGLLINKPROGRAMPROC				glLinkProgram = 0;
PFNGLGETPROGRAMINFOLOGPROC			glGetProgramInfoLog = 0;
PFNGLUSEPROGRAMPROC					glUseProgram = 0;
PFNGLGETSHADERIVPROC				glGetShaderiv = 0; 
PFNGLGETPROGRAMIVPROC				glGetProgramiv = 0;
PFNGLGETUNIFORMLOCATIONPROC			glGetUniformLocation = 0;
PFNGLUNIFORM4FPROC					glUniform4f = 0;
PFNGLUNIFORM1IPROC					glUniform1i = 0;
PFNGLACTIVETEXTUREPROC				glActiveTexture = 0;
#endif // __FRAGSHADER_NOARB

FORCE_INLINE void SkipExtension(uint8_t u8ExtensionType, xFileIO *pFileIO);

int xGraphicsInit(HWND hWnd, int iWindowWidth, int iWindowHeight, bool bFullscreenApp, int iBPP)
{
	PIXELFORMATDESCRIPTOR pfdpixeldesc; 
	int		ipixelformat;
	DEVMODE dmsettings;

	g_xGraphics.m_iScreenWidth	= iWindowWidth;
	g_xGraphics.m_iScreenHeight	= iWindowHeight;

	g_xGraphics.m_hWnd = hWnd;
	g_xGraphics.m_hDC  = NULL;
	g_xGraphics.m_hRC  = NULL;

	g_xGraphics.m_fGrayScaleFactor = 0.0f;

	g_xGraphics.m_iFrameSkip = 0;

	if(bFullscreenApp)
	{
		memset(&g_xGraphics.m_dmOldSettings, 0, sizeof(g_xGraphics.m_dmOldSettings));

		EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &g_xGraphics.m_dmOldSettings);

		memset(&dmsettings, 0, sizeof(dmsettings));				//Makes Sure Memory's Cleared
		
		// Get current settings -- This function fills our the settings
		// This makes sure NT and Win98 machines change correctly
		if(!EnumDisplaySettings(NULL, ENUM_CURRENT_SETTINGS, &dmsettings))
			return PE_RES_OGLFULLSCREENERROR;

		dmsettings.dmSize		= sizeof(dmsettings);
		dmsettings.dmPelsWidth	= g_xGraphics.m_iScreenWidth;
		dmsettings.dmPelsHeight	= g_xGraphics.m_iScreenHeight;
		dmsettings.dmBitsPerPel	= iBPP;
		dmsettings.dmFields		= DM_BITSPERPEL | DM_PELSWIDTH | DM_PELSHEIGHT;
		
		//This function actually changes the screen to full screen
		//CDS_FULLSCREEN Gets Rid Of Start Bar.
		if(ChangeDisplaySettings(&dmsettings, CDS_FULLSCREEN) != DISP_CHANGE_SUCCESSFUL)
			bFullscreenApp = false;
	}
	
	g_xGraphics.m_hDC = GetDC(g_xGraphics.m_hWnd);
	
	//Set BPP for the screen
	memset(&pfdpixeldesc, 0, sizeof(pfdpixeldesc));
	pfdpixeldesc.nSize			= sizeof(PIXELFORMATDESCRIPTOR);
	pfdpixeldesc.nVersion		= 1;													
	pfdpixeldesc.dwFlags		= PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER; 
	pfdpixeldesc.dwLayerMask	= PFD_MAIN_PLANE;
	pfdpixeldesc.iPixelType		= PFD_TYPE_RGBA;
	pfdpixeldesc.cColorBits		= iBPP;
	pfdpixeldesc.cDepthBits		= iBPP;
	pfdpixeldesc.cAccumBits		= 0;
	pfdpixeldesc.cStencilBits	= 0;
	
	//This gets us a pixel format that best matches the one passed in from the device
	if((ipixelformat = ChoosePixelFormat(g_xGraphics.m_hDC, &pfdpixeldesc)) == false)
		return PE_RES_OGLPIXELFORMATERROR;

	//This sets the pixel format that we extracted from above
	if(SetPixelFormat(g_xGraphics.m_hDC, ipixelformat, &pfdpixeldesc) == false)
		return PE_RES_OGLPIXELFORMATERROR;

	g_xGraphics.m_hRC = wglCreateContext(g_xGraphics.m_hDC);					//This creates a rendering context from our hdc
	wglMakeCurrent(g_xGraphics.m_hDC, g_xGraphics.m_hRC);						//This makes the rendering context we just created the one we want to use

	glViewport(0, 0, g_xGraphics.m_iScreenWidth, g_xGraphics.m_iScreenHeight);		// Reset The Current Viewport

	glMatrixMode(GL_PROJECTION);						// Select The Projection Matrix
	glLoadIdentity();									// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective(50.0f, (GLfloat)g_xGraphics.m_iScreenWidth / (GLfloat)g_xGraphics.m_iScreenHeight, 0.5f, 300.0f);

	glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
	glLoadIdentity();									// Reset The Modelview Matrix

	glShadeModel(GL_SMOOTH);
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClearDepth(1.0f);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);

	gluLookAt(0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 1.0, 0.0);
	glEnable(GL_TEXTURE_2D);
	
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);


#ifndef __FRAGSHADER_NOARB
	// OpenGL Extensions
	glGenProgramsARB				= (PFNGLGENPROGRAMSARBPROC)				wglGetProcAddress("glGenProgramsARB");
	glBindProgramARB				= (PFNGLBINDPROGRAMARBPROC)				wglGetProcAddress("glBindProgramARB");
	glProgramStringARB				= (PFNGLPROGRAMSTRINGARBPROC)			wglGetProcAddress("glProgramStringARB");
	glActiveTextureARB				= (PFNGLACTIVETEXTUREARBPROC)			wglGetProcAddress("glActiveTextureARB");
	glProgramLocalParameter4fARB	= (PFNGLPROGRAMLOCALPARAMETER4FARBPROC)	wglGetProcAddress("glProgramLocalParameter4fARB");
	glMultiTexCoord2fARB			= (PFNGLMULTITEXCOORD2FARBPROC)			wglGetProcAddress("glMultiTexCoord2fARB");

	if(glGenProgramsARB && glBindProgramARB && glProgramStringARB)
	{
		// Load Inversion Helper Shader
		char strShaderCode[10240] = {0};
		xFileIO fileio;

		if(fileio.OpenFile("shaders/AlphaKillPixel.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		glEnable(GL_FRAGMENT_PROGRAM_ARB);

		glGenProgramsARB(1, &g_xGraphics.m_uiAlphaKillPixelShaderIndex);
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiAlphaKillPixelShaderIndex);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, strlen(strShaderCode), strShaderCode);

		GLint err;
		char errorstring[256];
		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &err);
		if (err >= 0)
			sprintf(errorstring, "Error in fragment shader %s\n", glGetString(GL_PROGRAM_ERROR_STRING_ARB));

		fileio.CloseFile();

		// Load Alpha Mask Shader
		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/AlphaMaskPixel.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		glGenProgramsARB(1, &g_xGraphics.m_uiAlphaMaskPixelShaderIndex);
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiAlphaMaskPixelShaderIndex);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, strlen(strShaderCode), strShaderCode);

		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &err);
		if (err >= 0)
			sprintf(errorstring, "Error in fragment shader %s\n", glGetString(GL_PROGRAM_ERROR_STRING_ARB));

		fileio.CloseFile();

		// Load Gray Scale Shader
		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/GrayScalePixel.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		glGenProgramsARB(1, &g_xGraphics.m_uiGrayScalePixelShaderIndex);
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiGrayScalePixelShaderIndex);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, strlen(strShaderCode), strShaderCode);

		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &err);
		if (err >= 0)
			sprintf(errorstring, "Error in fragment shader %s\n", glGetString(GL_PROGRAM_ERROR_STRING_ARB));

		fileio.CloseFile();

		// Load Dual Color Shader
		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/DualColorFontPixel.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		glGenProgramsARB(1, &g_xGraphics.m_uiDualColorPixelShaderIndex);
		glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiDualColorPixelShaderIndex);
		glProgramStringARB(GL_FRAGMENT_PROGRAM_ARB, GL_PROGRAM_FORMAT_ASCII_ARB, strlen(strShaderCode), strShaderCode);

		glGetIntegerv(GL_PROGRAM_ERROR_POSITION_ARB, &err);
		if (err >= 0)
			sprintf(errorstring, "Error in fragment shader %s\n", glGetString(GL_PROGRAM_ERROR_STRING_ARB));

		glDisable(GL_FRAGMENT_PROGRAM_ARB);

		fileio.CloseFile();
	}
#else // __FRAGSHADER__NOARB
	// OpenGL Extensions
	glCreateShader					= (PFNGLCREATESHADERPROC)				wglGetProcAddress("glCreateShader");
	glShaderSource					= (PFNGLSHADERSOURCEPROC)				wglGetProcAddress("glShaderSource");
	glCompileShader					= (PFNGLCOMPILESHADERPROC)				wglGetProcAddress("glCompileShader");
	glGetShaderInfoLog				= (PFNGLGETSHADERINFOLOGPROC)			wglGetProcAddress("glGetShaderInfoLog");
	glCreateProgram					= (PFNGLCREATEPROGRAMPROC)				wglGetProcAddress("glCreateProgram");
	glAttachShader					= (PFNGLATTACHSHADERPROC)				wglGetProcAddress("glAttachShader");
	glLinkProgram					= (PFNGLLINKPROGRAMPROC)				wglGetProcAddress("glLinkProgram");
	glGetProgramInfoLog				= (PFNGLGETPROGRAMINFOLOGPROC)			wglGetProcAddress("glGetProgramInfoLog");
	glUseProgram					= (PFNGLUSEPROGRAMPROC)					wglGetProcAddress("glUseProgram");
	glGetShaderiv					= (PFNGLGETSHADERIVPROC)				wglGetProcAddress("glGetShaderiv");
	glGetProgramiv					= (PFNGLGETPROGRAMIVPROC)				wglGetProcAddress("glGetProgramiv");
	glGetUniformLocation			= (PFNGLGETUNIFORMLOCATIONPROC)			wglGetProcAddress("glGetUniformLocation");
	glUniform4f						= (PFNGLUNIFORM4FPROC)					wglGetProcAddress("glUniform4f");
	glUniform1i						= (PFNGLUNIFORM1IPROC)					wglGetProcAddress("glUniform1i");
	glActiveTexture					= (PFNGLACTIVETEXTUREPROC)				wglGetProcAddress("glActiveTexture");

	// Validate pointers
	if(	glCreateShader && glShaderSource && glCompileShader && glGetShaderInfoLog && glCreateProgram && glAttachShader &&
		glLinkProgram && glGetProgramInfoLog && glUseProgram && glGetShaderiv && glGetProgramiv && glGetUniformLocation &&
		glUniform4f && glUniform1i
		)
	{
		char errorstring[4608] = "";

		GLint compiled;

		// Load Inversion Helper Shader
		char strShaderCode[10240] = {0};
		const char *pShaderCode[1] = {(const char *)&strShaderCode}; 

		xFileIO fileio;

		if(fileio.OpenFile("shaders/AlphaKillPixelNoARB.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		// Create the shader object & compile it
		g_xGraphics.m_uiAlphaKillPixelShaderIndex = glCreateShader(GL_FRAGMENT_SHADER);
		glShaderSource(g_xGraphics.m_uiAlphaKillPixelShaderIndex, 1, pShaderCode, NULL);
		glCompileShader(g_xGraphics.m_uiAlphaKillPixelShaderIndex);

		// Validate compilation
		glGetShaderiv(g_xGraphics.m_uiAlphaKillPixelShaderIndex, GL_COMPILE_STATUS, &compiled ); //
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetShaderInfoLog(g_xGraphics.m_uiAlphaKillPixelShaderIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment shader AlphaKillPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		// Create the program, attach the shader and link it
		g_xGraphics.m_uiAlphaKillPixelProgramIndex = glCreateProgram();
		glAttachShader(g_xGraphics.m_uiAlphaKillPixelProgramIndex, g_xGraphics.m_uiAlphaKillPixelShaderIndex);
		glLinkProgram(g_xGraphics.m_uiAlphaKillPixelProgramIndex);

		// Validate linkage
		glGetProgramiv(g_xGraphics.m_uiAlphaKillPixelProgramIndex, GL_LINK_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetProgramInfoLog(g_xGraphics.m_uiAlphaKillPixelProgramIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment program AlphaKillPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		// Get the location of the arguments
		g_xGraphics.m_xAlphaKillPixel.m_iTexture = glGetUniformLocation(g_xGraphics.m_uiAlphaKillPixelProgramIndex, "tex");

		fileio.CloseFile();

		GLint err;
		// Load Alpha Mask Shader
		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/AlphaMaskPixelNoARB.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		g_xGraphics.m_uiAlphaMaskPixelShaderIndex = glCreateShader(GL_FRAGMENT_SHADER);
		glShaderSource(g_xGraphics.m_uiAlphaMaskPixelShaderIndex, 1, pShaderCode, NULL);
		glCompileShader(g_xGraphics.m_uiAlphaMaskPixelShaderIndex);

		glGetShaderiv(g_xGraphics.m_uiAlphaMaskPixelShaderIndex, GL_COMPILE_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetShaderInfoLog(g_xGraphics.m_uiAlphaMaskPixelShaderIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment shader AlphaMaskPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_uiAlphaMaskPixelProgramIndex = glCreateProgram();
		glAttachShader(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, g_xGraphics.m_uiAlphaMaskPixelShaderIndex);
		glLinkProgram(g_xGraphics.m_uiAlphaMaskPixelProgramIndex);

		glGetProgramiv(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, GL_LINK_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetProgramInfoLog(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment program AlphaMaskPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_xAlphaMaskPixel.m_iTextureRatio = glGetUniformLocation(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, "texratio");
		g_xGraphics.m_xAlphaMaskPixel.m_iTextureOffset = glGetUniformLocation(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, "texoffset");
		g_xGraphics.m_xAlphaMaskPixel.m_iGrayScaleFactor = glGetUniformLocation(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, "grayScaleFactor");
		g_xGraphics.m_xAlphaMaskPixel.m_iMask = glGetUniformLocation(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, "mask");
		g_xGraphics.m_xAlphaMaskPixel.m_iTexture = glGetUniformLocation(g_xGraphics.m_uiAlphaMaskPixelProgramIndex, "tex");

		fileio.CloseFile();

		// Load Gray Scale Shader
		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/GrayScalePixelNoARB.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		g_xGraphics.m_uiGrayScalePixelShaderIndex = glCreateShader(GL_FRAGMENT_SHADER);
		glShaderSource(g_xGraphics.m_uiGrayScalePixelShaderIndex, 1, pShaderCode, NULL);
		glCompileShader(g_xGraphics.m_uiGrayScalePixelShaderIndex);

		glGetShaderiv(g_xGraphics.m_uiGrayScalePixelShaderIndex, GL_COMPILE_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetShaderInfoLog(g_xGraphics.m_uiGrayScalePixelShaderIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment shader GrayScalePixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_uiGrayScalePixelProgramIndex = glCreateProgram();
		glAttachShader(g_xGraphics.m_uiGrayScalePixelProgramIndex, g_xGraphics.m_uiGrayScalePixelShaderIndex);
		glLinkProgram(g_xGraphics.m_uiGrayScalePixelProgramIndex);

		glGetProgramiv(g_xGraphics.m_uiGrayScalePixelProgramIndex, GL_LINK_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetProgramInfoLog(g_xGraphics.m_uiGrayScalePixelProgramIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment program GrayScalePixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_xGrayScalePixel.m_iGrayScaleFactor = glGetUniformLocation(g_xGraphics.m_uiGrayScalePixelProgramIndex, "grayScaleFactor");
		g_xGraphics.m_xGrayScalePixel.m_iTexture = glGetUniformLocation(g_xGraphics.m_uiGrayScalePixelProgramIndex, "tex");

		fileio.CloseFile();

		// Load Dual Color Shader

		memset(strShaderCode, 0, 10240);

		if(fileio.OpenFile("shaders/DualColorFontPixelNoARB.pix", true, false))
			return PE_RES_FILEOPENERROR;

		if(fileio.ReadToEnd(strShaderCode))
			return PE_RES_FILEREADERROR;

		g_xGraphics.m_uiDualColorPixelShaderIndex = glCreateShader(GL_FRAGMENT_SHADER);
		glShaderSource(g_xGraphics.m_uiDualColorPixelShaderIndex, 1, pShaderCode, NULL);
		glCompileShader(g_xGraphics.m_uiDualColorPixelShaderIndex);

		glGetShaderiv(g_xGraphics.m_uiDualColorPixelShaderIndex, GL_COMPILE_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetShaderInfoLog(g_xGraphics.m_uiDualColorPixelShaderIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment shader DualColorPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_uiDualColorPixelProgramIndex = glCreateProgram();
		glAttachShader(g_xGraphics.m_uiDualColorPixelProgramIndex, g_xGraphics.m_uiDualColorPixelShaderIndex);
		glLinkProgram(g_xGraphics.m_uiDualColorPixelProgramIndex);

		glGetProgramiv(g_xGraphics.m_uiDualColorPixelProgramIndex, GL_LINK_STATUS, &compiled );
		if(compiled == GL_FALSE)
		{
			char strBuf[4096] = "";
			glGetProgramInfoLog(g_xGraphics.m_uiDualColorPixelProgramIndex, 4096, NULL, strBuf);
			sprintf_s(errorstring, 4608, "Error in fragment program DualColorPixel:\n%s", strBuf);
			MessageBox(0, errorstring, "Failure", MB_OK);
			return PE_RES_FRAGMENTERROR;
		}

		g_xGraphics.m_xDualColorPixel.m_iSecondColor = glGetUniformLocation(g_xGraphics.m_uiDualColorPixelProgramIndex, "secondColor");
		g_xGraphics.m_xDualColorPixel.m_iColorDifference = glGetUniformLocation(g_xGraphics.m_uiDualColorPixelProgramIndex, "colorDifference");
		g_xGraphics.m_xDualColorPixel.m_iGrayScaleFactor = glGetUniformLocation(g_xGraphics.m_uiDualColorPixelProgramIndex, "grayScaleFactor");
		g_xGraphics.m_xDualColorPixel.m_iTexture = glGetUniformLocation(g_xGraphics.m_uiDualColorPixelProgramIndex, "tex");

		fileio.CloseFile();
	}

#endif // __FRAGSHADER__NOARB

	return PE_RES_NOERROR;
}

void xGraphicsPreUpdate(DWORD dwTime)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
	glLoadIdentity();									// Reset The Current Modelview Matrix
}

void xGraphicsUpdate(double dTime)
{
	xRenderDataShader *shaderdata;
	xRenderDataTexture *texturedata;
	xRenderDataVerts *vertdata;
	bool is3d = 0;
	bool outlinepass = 0, outlinedone = 0;
		
	shaderdata = g_xGraphics.m_pSceneGraph->m_xRenderData.m_pNext;

	if(g_xGraphics.m_iFrameSkip <= 0)
	{
		glEnable(GL_BLEND);
		glEnableClientState(GL_VERTEX_ARRAY);

		while(shaderdata)
		{
			if(shaderdata->m_bIs3D != is3d)
			{
				if(shaderdata->m_bIs3D)
				{
					xGraphicsReset2DMode();
					is3d = true;
				}
				else
				{
					xGraphicsGoto2DMode();
					is3d = false;
				}
			}

			if(shaderdata->m_bRender)
			{
				switch(shaderdata->m_iBlendMode)
				{
					case PE_BLEND_MODE_NONE:
#ifndef __FRAGSHADER_NOARB
							glEnable(GL_FRAGMENT_PROGRAM_ARB);
							glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiGrayScalePixelShaderIndex);
							glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_ALPHA_GREYSCALE, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#else // __FRAGSHADER_NOARB
							// Load the program and set the grayScaleFactor value
							glUseProgram( g_xGraphics.m_uiGrayScalePixelProgramIndex );
							glUniform4f( g_xGraphics.m_xGrayScalePixel.m_iGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#endif // __FRAGSHADER_NOARB
							glDisable(GL_BLEND);
							glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
						break;

					case PE_BLEND_MODE_NONE_OUTLINED:
						if(outlinedone)
						{
							outlinepass = 0;
							glDisable(GL_LINE_SMOOTH);
							glEnable(GL_DEPTH_TEST);
#ifndef __FRAGSHADER_NOARB
							glEnable(GL_FRAGMENT_PROGRAM_ARB);
							glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiGrayScalePixelShaderIndex);
							glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_ALPHA_GREYSCALE, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#else // __FRAGSHADER_NOARB
							// Load the program and set the grayScaleFactor value
							glUseProgram( g_xGraphics.m_uiGrayScalePixelProgramIndex );
							glUniform4f( g_xGraphics.m_xGrayScalePixel.m_iGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#endif // __FRAGSHADER_NOARB
							glDisable(GL_BLEND);
							glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
							glLineWidth(1.0f);
						}
						else
						{
							outlinepass = 1;
							glDisable(GL_DEPTH_TEST);
#ifndef __FRAGSHADER_NOARB
							glDisable(GL_FRAGMENT_PROGRAM_ARB);
#endif // __FRAGSHADER_NOARB
							glEnable(GL_LINE_SMOOTH);
							glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
							glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
							glLineWidth(3.0f);
						}
						break;

					case PE_BLEND_MODE_ALPHA:
#ifndef __FRAGSHADER_NOARB
						glEnable(GL_FRAGMENT_PROGRAM_ARB);
						glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiGrayScalePixelShaderIndex);
						glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_ALPHA_GREYSCALE, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#else // __FRAGSHADER_NOARB
						// Load the program and set the grayScaleFactor value
						glUseProgram( g_xGraphics.m_uiGrayScalePixelProgramIndex );
						glUniform4f( g_xGraphics.m_xGrayScalePixel.m_iGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#endif // __FRAGSHADER_NOARB
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;

					case PE_BLEND_MODE_ALPHA_NOSHADER:
#ifndef __FRAGSHADER_NOARB
						glDisable(GL_FRAGMENT_PROGRAM_ARB);
#endif // __FRAGSHADER_NOARB
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;

					case PE_BLEND_MODE_INVERTED_COLOR:
#ifndef __FRAGSHADER_NOARB
						glEnable(GL_FRAGMENT_PROGRAM_ARB);
						glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiAlphaKillPixelShaderIndex);
#else // __FRAGSHADER_NOARB
						// Load the program
						glUseProgram(g_xGraphics.m_uiAlphaKillPixelProgramIndex);
#endif // __FRAGSHADER_NOARB
						glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
						break;
			
					case PE_BLEND_MODE_MASKED:
#ifndef __FRAGSHADER_NOARB
						glEnable(GL_FRAGMENT_PROGRAM_ARB);
						glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiAlphaMaskPixelShaderIndex);
						glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_MASKED_GREYSCALE, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#else // __FRAGSHADER_NOARB
						// Load the program and set the grayScaleFactor value
						glUseProgram( g_xGraphics.m_uiAlphaMaskPixelProgramIndex );
						glUniform4f( g_xGraphics.m_xAlphaMaskPixel.m_iGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0 );
#endif // __FRAGSHADER_NOARB
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;

					case PE_BLEND_MODE_2COLOR:
#ifndef __FRAGSHADER_NOARB
						glEnable(GL_FRAGMENT_PROGRAM_ARB);
						glBindProgramARB(GL_FRAGMENT_PROGRAM_ARB, g_xGraphics.m_uiDualColorPixelShaderIndex);
						glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_2COLOR_GREYSCALE, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0);
#else // __FRAGSHADER_NOARB
						// Load the program and set the grayScaleFactor value
						glUseProgram( g_xGraphics.m_uiDualColorPixelProgramIndex );
						glUniform4f( g_xGraphics.m_xDualColorPixel.m_iGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, g_xGraphics.m_fGrayScaleFactor, 0.0 );
#endif // __FRAGSHADER_NOARB
						glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
						break;
				}

				texturedata = shaderdata->m_xTextureDataList.m_pNext;

				while(texturedata)
				{
					if(texturedata->m_bRender)
					{
						switch(shaderdata->m_iBlendMode)
						{
							case PE_BLEND_MODE_MASKED:
#ifndef __FRAGSHADER_NOARB
								glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_MASKED_TEXTUREOFFSET, 
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].R,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].G,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].B,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].A);

								glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_MASKED_TEXTURERATIO, 
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].R,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].G,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].B,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].A);

								glActiveTextureARB(GL_TEXTURE1_ARB);
								glEnable(GL_TEXTURE_2D);
								glBindTexture(GL_TEXTURE_2D, texturedata->m_uiTextureIndexArray[1]);

								glActiveTextureARB(GL_TEXTURE0_ARB);
#else // __FRAGSHADER_NOARB
								// set the grayScaleFactor value
								glUniform4f( g_xGraphics.m_xAlphaMaskPixel.m_iTextureOffset,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTUREOFFSET].A);

								glUniform4f( g_xGraphics.m_xAlphaMaskPixel.m_iTextureRatio,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_MASKED_TEXTURERATIO].A);

								glActiveTexture(GL_TEXTURE0);
								glBindTexture(GL_TEXTURE_2D,texturedata->m_uiTextureIndexArray[1]);
								glUniform1i(g_xGraphics.m_xAlphaMaskPixel.m_iMask, 0);

								glActiveTexture(GL_TEXTURE1);
#endif // __FRAGSHADER_NOARB
								break;

							case PE_BLEND_MODE_2COLOR:
#ifndef __FRAGSHADER_NOARB
								glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_2COLOR_COLOR2, 
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].R,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].G,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].B,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].A);

								glProgramLocalParameter4fARB(GL_FRAGMENT_PROGRAM_ARB, PE_SHADERPARAM_2COLOR_COLORDIFF, 
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].R,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].G,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].B,
															 texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].A);
#else // __FRAGSHADER_NOARB
								glUniform4f( g_xGraphics.m_xDualColorPixel.m_iSecondColor,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLOR2].A);
								glUniform4f( g_xGraphics.m_xDualColorPixel.m_iColorDifference,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].R,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].G,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].B,
											texturedata->m_xShaderParameterArray[PE_SHADERPARAM_2COLOR_COLORDIFF].A);
#endif // __FRAGSHADER_NOARB
								break;
						}

						if(texturedata->m_iTextureIndexArrayLength > 0)
						{
#ifndef __FRAGSHADER_NOARB
							glEnable(GL_TEXTURE_2D);
#endif // __FRAGSHADER_NOARB

							glBindTexture(GL_TEXTURE_2D, texturedata->m_uiTextureIndexArray[0]);

#ifdef __FRAGSHADER_NOARB
							if(shaderdata->m_iBlendMode == PE_BLEND_MODE_MASKED)
								glUniform1i(g_xGraphics.m_xAlphaMaskPixel.m_iTexture,1);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_INVERTED_COLOR)
								glUniform1i(g_xGraphics.m_xAlphaKillPixel.m_iTexture, 0);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_2COLOR)
								glUniform1i(g_xGraphics.m_xDualColorPixel.m_iTexture, 0);
							else if(shaderdata->m_iBlendMode == PE_BLEND_MODE_NONE_OUTLINED || shaderdata->m_iBlendMode == PE_BLEND_MODE_ALPHA)
								glUniform1i(g_xGraphics.m_xGrayScalePixel.m_iTexture, 0);
#endif // __FRAGSHADER_NOARB
							glEnableClientState(GL_TEXTURE_COORD_ARRAY);
						}
						else
						{
#ifndef __FRAGSHADER_NOARB
							glDisable(GL_TEXTURE_2D); // fragment shader overrides this
#endif // __FRAGSHADER_NOARB
							glDisableClientState(GL_TEXTURE_COORD_ARRAY);

							if(texturedata->m_uiRenderType == GL_LINES || texturedata->m_uiRenderType == GL_LINE_LOOP || texturedata->m_uiRenderType == GL_LINE_STRIP)
								glLineWidth(texturedata->m_fLineWidth);
						}

						vertdata = texturedata->m_xVertDataList.m_pNext;

						while(vertdata)
						{
							if(vertdata->m_bRender)
							{
								if(shaderdata->m_bIs3D)
								{
									glPushMatrix();
									glRotatef(vertdata->m_xRotation.x, 1.0, 0.0, 0.0);
									glRotatef(vertdata->m_xRotation.y, 0.0, 1.0, 0.0);
									glRotatef(vertdata->m_xRotation.z, 0.0, 0.0, 1.0);
								}
								else if(vertdata->m_xRotation.x)
								{
									glPushMatrix();
									glRotatef(vertdata->m_xRotation.x, 1.0, 0.0, 0.0);
								}

								if(outlinepass)
									glColor4f(0.0, 0.0, 0.0, 1.0);
	/*							else if(vertdata->m_pColorArray)
								{
									glEnableClientState(GL_COLOR_ARRAY);
									glColorPointer(4, GL_FLOAT, 0, vertdata->m_pColorArray);
								}
	*/							else
									glColor4f(vertdata->m_xColor.R, vertdata->m_xColor.G, vertdata->m_xColor.B, vertdata->m_xColor.A);

								if(texturedata->m_iTextureIndexArrayLength > 0)
									glTexCoordPointer(2, GL_FLOAT, 0, vertdata->m_xTextureCoordinatesArray);

								glVertexPointer(3, GL_FLOAT, 0, vertdata->m_xVectorArray);

								glDrawArrays(texturedata->m_uiRenderType, 0, vertdata->m_iVectorArrayLength);

	//							if(vertdata->m_pColorArray)
	//								glDisableClientState(GL_COLOR_ARRAY);

								if(shaderdata->m_bIs3D || vertdata->m_xRotation.x)
									glPopMatrix();
							}

							vertdata = vertdata->m_pNext;
						}
					}

					texturedata = texturedata->m_pNext;
				}

				switch(shaderdata->m_iBlendMode)
				{
					case PE_BLEND_MODE_NONE:
						glEnable(GL_BLEND);
						break;

					case PE_BLEND_MODE_NONE_OUTLINED:
						if(outlinedone)
						{
							glEnable(GL_BLEND);
							outlinedone = 0;
						}
						else
							outlinedone = 1;
						break;

					case PE_BLEND_MODE_MASKED:
#ifndef __FRAGSHADER_NOARB
						glActiveTextureARB(GL_TEXTURE1_ARB);
						glDisable(GL_TEXTURE_2D);
						glActiveTextureARB(GL_TEXTURE0_ARB);
#else
						glActiveTexture(GL_TEXTURE1);
						glDisable(GL_TEXTURE_2D);
						glActiveTexture(GL_TEXTURE0);
#endif
						break;
				}
			}

			if(!outlinepass)
				shaderdata = shaderdata->m_pNext;
		}

		glColor4f(1.0, 1.0, 1.0, 1.0);
		glDisableClientState(GL_VERTEX_ARRAY);
		glDisable(GL_BLEND);

		SwapBuffers(g_xGraphics.m_hDC);
	}
	else
		g_xGraphics.m_iFrameSkip--;
}

void xGraphicsShutdown()
{
	wglMakeCurrent(NULL, NULL);
	wglDeleteContext(g_xGraphics.m_hRC);
	ReleaseDC(g_xGraphics.m_hWnd, g_xGraphics.m_hDC);
}

void xGraphicsGetDesktopResolution(int *pScreenWidth, int *pScreenHeight)
{
	*pScreenWidth = GetSystemMetrics(SM_CXSCREEN);
	*pScreenHeight = GetSystemMetrics(SM_CYSCREEN);
}

void xGraphicsStartGettingAllDesktopResolutions()
{
	g_xGraphics.m_iAllScreenResolutionsIndex = 0;
	g_xGraphics.m_dwDisplayFrequency = 0;
}

bool xGraphicsGetNextDesktopResolution(int *pScreenWidth, int *pScreenHeight)
{
	DEVMODE devmode;

	memset(&devmode, 0, sizeof(devmode));
	devmode.dmSize = sizeof(devmode);

	while((g_xGraphics.m_dwDisplayFrequency != 0 && g_xGraphics.m_dwDisplayFrequency != devmode.dmDisplayFrequency) || devmode.dmBitsPerPel != 32 || devmode.dmDefaultSource != 0)
	{
		if(!EnumDisplaySettings(NULL, g_xGraphics.m_iAllScreenResolutionsIndex, &devmode))
			return 0;

		g_xGraphics.m_iAllScreenResolutionsIndex++;
	}

	g_xGraphics.m_dwDisplayFrequency = devmode.dmDisplayFrequency;

	*pScreenWidth = devmode.dmPelsWidth;
	*pScreenHeight = devmode.dmPelsHeight;

	return 1;
}

bool xGraphicsIsResolutionAvailable(int iScreenWidth, int iScreenHeight, bool bFullscreen)
{
	DEVMODE devmode;
	int i, j;

	if(bFullscreen)
	{
		memset(&devmode, 0, sizeof(devmode));
		devmode.dmSize = sizeof(devmode);

		for(i = 0; EnumDisplaySettings(NULL, i, &devmode); i++)
		{
			if(devmode.dmPelsWidth == iScreenWidth && devmode.dmPelsHeight == iScreenHeight)
				return 1;
		}
	}
	else
	{
		xGraphicsGetDesktopResolution(&i, &j);
		return iScreenWidth < i && iScreenHeight < j;
	}

	return 0;
}

xTexture* xGraphicsGetTexture(const char* strFilePath, int iFrame)
{
	int i;
	char strFile[256];
	xTexture *temptexture, *ret;
	xFileIO objectfile;
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;

	sprintf(strFile, "%s%i", strFilePath, iFrame);

	ret = xGraphicsTexturePoolNew(strFile);

	if(!ret->m_bLoaded)
	{
		if(objectfile.OpenFile(strFilePath, true, true))
			assert(0);

		if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
			assert(0);

		if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
			assert(0);

		for(i = 0; i < spriteheader.m_iNumFrames; i++)
		{
			if(i == iFrame)
			{
				xTextureCreateTexture(ret, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				strcpy(ret->m_strObjectPath, strFile);
				ret->m_bLoaded = true;
			}
			else
			{
				sprintf(strFile, "%s%i", strFilePath, i);
				temptexture = xGraphicsTexturePoolNew(strFile);
				xTextureCreateTexture(temptexture, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				strcpy(temptexture->m_strObjectPath, strFile);
				temptexture->m_bLoaded = true;
			}

			objectfile.ReadData(&temprect, sizeof(RECT));
		}

		objectfile.CloseFile();
	}

	return ret;
}

int xGraphicsGetTextures(const char* strFilePath, xTexture ***pTextures, float *fAnimationRate)
{
	int i;
	char strFile[256];
	xTexture *temptextures[256];
	xFileIO objectfile;
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;

	sprintf(strFile, "%s0", strFilePath);

	temptextures[0] = xGraphicsTexturePoolNew(strFile);

	if(objectfile.OpenFile(strFilePath, true, true))
		assert(0);

	if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
		assert(0);

	if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
		assert(0);

	if(!temptextures[0]->m_bLoaded)
	{
		*pTextures = (xTexture**)dlmalloc(sizeof(xTexture*) * spriteheader.m_iNumFrames);
		(*pTextures)[0] = temptextures[0];

		for(i = 0; i < spriteheader.m_iNumFrames; i++)
		{
			if(i > 0)
			{
				sprintf(strFile, "%s%i", strFilePath, i);
				(*pTextures)[i] = xGraphicsTexturePoolNew(strFile);
			}

			xTextureCreateTexture((*pTextures)[i], &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
			strcpy((*pTextures)[i]->m_strObjectPath, strFile);
			(*pTextures)[i]->m_bLoaded = true;

			objectfile.ReadData(&temprect, sizeof(RECT));
		}
	}
	else
	{
		i = 1;
		while(1)
		{
			sprintf(strFile, "%s%i", strFilePath, i);
			temptextures[i] = xGraphicsTexturePoolGet(strFile);

			if(!temptextures[i])
				break;

			i++;
		}

		*pTextures = (xTexture**)dlmalloc(sizeof(xTexture*) * i);

		for(spriteheader.m_iNumFrames = 0; spriteheader.m_iNumFrames < i; spriteheader.m_iNumFrames++)
			(*pTextures)[spriteheader.m_iNumFrames] = temptextures[spriteheader.m_iNumFrames];
	}

	objectfile.CloseFile();

	if(fAnimationRate)
		*fAnimationRate = 1.0f / (float)spriteheader.m_iFPS;

	return spriteheader.m_iNumFrames;
}

void xGraphicsReloadTexturePool()
{
	xTexture *texture, *temptexture;
	xFileIO objectfile;
	char texturepath[256], temppath[256], framestring[4];
	xP2DFileHeader fileheader;
	xP2DSpriteHeader spriteheader;
	RECT temprect;
	uint8_t extensiontype;
	int frame, i, j;

	texture = g_xGraphics.m_xTexturePool.m_pNext;

	while(texture)
	{
		strcpy(texturepath, texture->m_strObjectPath);

		for(i = strlen(texturepath) - 1; i > 0; i--)
		{
			if(texturepath[i] == '.')
			{
				framestring[0] = texturepath[i + 4];

				if(framestring[0] != '\0')
				{
					framestring[1] = texturepath[i + 5];

					if(framestring[1] != '\0')
					{
						framestring[2] = texturepath[i + 6];
						framestring[3] = '\0';
					}

					frame = atoi(framestring);
				}
				else
					frame = 0;

				texturepath[i + 4] = '\0';
				break;
			}
		}

		if(objectfile.OpenFile(texturepath, true, true))
			assert(0);

		if(objectfile.ReadData(&fileheader, sizeof(xP2DFileHeader)))
			assert(0);

		for(j = 0; j < fileheader.m_iNumSprites; j++)
		{
			if(objectfile.ReadData(&spriteheader, sizeof(xP2DSpriteHeader)))
				assert(0);

			while(spriteheader.m_iNumExtensions > 0)
			{
				objectfile.ReadData(&extensiontype, sizeof(uint8_t));

				SkipExtension(extensiontype, &objectfile);

				spriteheader.m_iNumExtensions--;
			}

			for(i = 0; i < spriteheader.m_iNumFrames; i++)
			{
				if(i + j == frame)
				{
					xTextureCreateTexture(texture, &objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				}
				else
				{
					xTextureSkipTexture(&objectfile, spriteheader.m_iWidth, spriteheader.m_iHeight);
				}

				objectfile.ReadData(&temprect, sizeof(RECT));
			}
		}

		objectfile.CloseFile();

		texture = texture->m_pNext;
	}
}

FORCE_INLINE void SkipExtension(uint8_t u8ExtensionType, xFileIO *pFileIO)
{
	uint8_t tempint8array[3];
	uint16_t tempint16;

	switch(u8ExtensionType)
	{
		case PE_P2D_EXTTYPE_PARTICLE:
		{
			// Skip ahead to the colors(because they are variable sized)
			pFileIO->SkipAhead(73);

			// Get the number of colors
			pFileIO->ReadData(&tempint16, sizeof(uint16_t));

			// For each color, read in the R, G, and B values(1 byte each)
			while(tempint16 > 0)
			{
				pFileIO->ReadData((void*)&tempint8array, sizeof(uint8_t) * 3);
				tempint16--;
			}

			break;
		}
	}
}

void xGraphicsDeleteTexture(xTexture* pTexture)
{
	xTexture *current;

	if(pTexture->m_iReferenceCount == 0)
	{
		GLuint texids[1] = {pTexture->m_uiID};
		glDeleteTextures(1, texids);

		// Remove it from the Texture Pool
		current = g_xGraphics.m_xTexturePool.m_pNext;
		while(current)
		{
			if(current == pTexture)
			{
				if(current->m_pPrev)
					current->m_pPrev->m_pNext = current->m_pNext;

				if(current->m_pNext)
					current->m_pNext->m_pPrev = current->m_pPrev;

				dlfree(pTexture);

				break;
			}

			current = current->m_pNext;
		}
	}
	else
		pTexture->m_iReferenceCount--;
}

void xGraphicsDeleteTextures(xTexture **pTextures, int iCount)
{
	int i;

	for(i = 0; i < iCount; i++)
		xGraphicsDeleteTexture(pTextures[i]);

	dlfree(pTextures);
}
