#ifndef MD3STRUCTS_H_
#define MD3STRUCTS_H_

#include "xVector.h"

struct xMD3Header
{
	char	m_strFileID[4];	// File ID - Must be "IDP3"
	int		m_iVersion;		// File version - Must be 15
	char	m_strFile[68];	// Name of the file
	int		m_iNumFrames;	// Number of animation frames
	int		m_iNumTags;		// Tag count
	int		m_iNumMeshes;	// Number of sub-objects in the mesh
	int		m_iNumMaxSkins;	// Number of skins for the mesh
	int		m_iHeaderSize;	// Mesh header size
	int		m_iTagStart;	// Offset into the file for tags
	int		m_iTagEnd;		// End offset into the file for tags
	int		m_iFileSize;	// File size
};

struct xMD3MeshInfo
{
	char	m_strMeshID[4];		// Mesh ID (We don't care)
	char	m_strName[68];		// Mesh name (We do care)
	int		m_iNumMeshFrames;	// Mesh aniamtion frame count
	int		m_iNumSkins;		// Mesh skin count
	int     m_iNumVertices;		// Mesh vertex count
	int		m_iNumTriangles;	// Mesh face count
	int		m_iTriangleStart;	// Starting offset for the triangles
	int		m_iHeaderSize;		// Header size for the mesh
	int     m_iUVStart;			// Starting offset for the UV coordinates
	int		m_iVertexStart;		// Starting offset for the vertex indices
	int		m_iMeshSize;		// Total mesh size
};

// This is our tag structure for the .MD3 file format.  These are used link other
// models to and the rotate and translate the child models of that model.
struct xMD3Tag
{
	char	m_strName[64];		// Name of the tag (I.E. "tag_torso")
	xVector	m_vecPosition;		// Translation that should be performed
	float	m_fRotation[3][3];	// 3x3 rotation matrix for this frame
};

// This stores the bone information (useless as far as I can see...)
struct xMD3Bone
{
	float	m_fMins[3];			// Min (x, y, z) value for the bone
	float	m_fMaxes[3];		// Max (x, y, z) value for the bone
	float	m_fPosition[3];		// Supposed the bone position???
	float	m_fScale;			// Scale of the bone
	char	m_strCreator[16];	// Modeler used to create the model (I.E. "3DS Max")
};

// This stores the normals and vertex indices 
struct xMD3Triangle
{
   signed short	 m_s8Vertex[3];				// The vertex for this face (scale down by 64.0f)
   unsigned char m_u8Normal[2];				// This stores some crazy normal values (not sure...)
};

// This stores the indices into the vertex and texture coordinate arrays
struct xMD3Face
{
   int m_iVertexIndices[3];				
};

// This stores UV coordinates
struct xMD3TexCoord
{
   float m_fTextureCoord[2];
};

// This stores a skin name (We don't use this, just the name of the model to get the texture)
struct xMD3Skin
{
	char m_strName[68];
};

struct xMD3Animation
{
	int m_iFirstFrame;
	int m_iLastFrame;
	int m_iNumBeats;
};

#endif