/////////////////////////////////////////////////////////////////
// Name: xMenu.h
/////////////////////////////////////////////////////////////////
// Description:
// - Serves as an object for each seperate menu.
/////////////////////////////////////////////////////////////////
// - 7.15.2004, Update, Dayle Flowers(Xienen)
//   File Created
/////////////////////////////////////////////////////////////////
#ifndef XMENU_H_
#define XMENU_H_

#include "xKeyFrame.h"
#include "xParser.h"
#include "xGameDefines.h"
#include "xGameOptions.h"

enum xDragType
{
	BB_DRAGTYPE_NONE = 0,
	BB_DRAGTYPE_PLAYERMOVEICON,
	BB_DRAGTYPE_SELECTEDPLAYERMOVEICON,
	BB_DRAGTYPE_MAX
};

struct xPingInfo
{
	int		m_iRequestCount;
	int		m_iResponseCount;
	DWORD	m_dwPingTotal;
};

enum xSinglePlayerLogin
{
	BB_SPLOGIN_MULTIPLAYER = 0,
	BB_SPLOGIN_SINGLEPLAYER,
	BB_SPLOGIN_CHALLENGES,
	BB_SPLOGIN_FREEPLAY,
	BB_SPLOGIN_CHANGELOGIN
};

//GUI Callbacks
void xMenuRunPostLoadCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow);
void xMenuRunInputCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xMenuRunKeyCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xMenuRunHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xGUIObject* &pOutHoverObject);
void xMenuRunUnHoverCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xMenuRunDragCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject, xRenderable2DQuad* &pOutDragObject);
void xMenuRunDropCommand(char *strCommand, char **strParams, xGUIWindowList *pWindow, xGUIObject *pObject);
void xMenuHandlePaste(char *strContents, xGUIWindowList *pWindow, xGUIObject *pObject);

class xMenu : public xGameState
{
public:
	xMenu()
	{
		m_bSlideTransitioning		= 0;
		m_bPingingServers			= 0;
		m_pPingTimes				= 0;
		m_iPingTimesLength			= 1024;
		m_pPingInfo					= 0;
		m_iSongDifficulty			= 1;
		m_pAudioStream				= 0;
		m_dStartGameCountdown		= 0.0;
		m_iBindingKeyIndex			= -1;
		m_xSinglePlayerLogin		= BB_SPLOGIN_MULTIPLAYER;
		m_pAchievementIcon[0]		= 0;
		m_strChallengeSongNames[0]	= 0;
		m_strFreePlaySongList		= 0;
		m_iFreePlayTopSong			= 0;
		m_iFreePlayDifficulty		= 1;
		m_iFreePlayExtraBlockCount	= 0;
		m_iFreePlayEnemy			= 0;
		m_iFreePlayEnemyCount		= 0;
		m_iFreePlayScrollAmount		= 0;
		m_fFreePlayScrollTime		= BB_FREEPLAY_SCROLLTIME;
		m_fSPDifficultyOriginalY	= 0.0f;

		m_xGUI.pfnRunPostLoadCommand	= &xMenuRunPostLoadCommand;
		m_xGUI.pfnRunInputCommand		= &xMenuRunInputCommand;
		m_xGUI.pfnRunKeyCommand			= &xMenuRunKeyCommand;
		m_xGUI.pfnRunHoverCommand		= &xMenuRunHoverCommand;
		m_xGUI.pfnRunUnHoverCommand		= &xMenuRunUnHoverCommand;
		m_xGUI.pfnRunDragCommand		= &xMenuRunDragCommand;
		m_xGUI.pfnRunDropCommand		= &xMenuRunDropCommand;
		m_xGUI.pfnHandlePaste			= &xMenuHandlePaste;
	}

	bool Init(const char *strFile);
	bool PreUpdate(DWORD dwTime);
	bool Update(double dTime);
	void RefreshAllRenderables();
	void Shutdown();

	void PlayMusic() { m_pAudioStream->SetTime(0); m_pAudioStream->Play(); }
	void StopMusic() { m_pAudioStream->Stop(); }

	bool AddMenuWindow(const char *strWindowName);
	void AddMainMenu();
	void SlideTransition(bool bToRightWindow, xGUIWindow *pLeftWindow, xGUIWindow *pRightWindow);
	void RemoveAllMenuWindows();

	void StartGame(char* strLevel);
	void InitializeBattlesWindow(xGUIWindowList* pMenuWindow);
	void SelectTour(xGUIWindowList *pWindow, char* strTour, char* strTourBackground = 0);
	void InitializeChallengesWindow(xGUIWindowList *pWindow);
	void ChallengesScroll(int iAmount);
	void ChallengeStart(int iChallenge);
	void InitializeFreePlayWindow(xGUIWindowList *pWindow);
	void FreePlayScrollSongs(int iAmount);
	void InitializeOptionsWindow(xGUIWindowList *pWindow);
	void InitializeHostServerWindow(xGUIWindowList *pWindow);

	void RefreshServerList();
	void OnFailedToInitListenServer(int iPort);

	void Login(char *strUser, char *strPass, bool bPassEncrypted, xGUIWindowList *pWindow = 0);
	void InitPingInfo();
	void PingServer(char* strAddress, int iPort, int iListNumber);
	void RepingServers();
	void HandlePingPackets();
	void ClearUnresponsiveServers();

	void LoadAndValidateCDKey();
	void StoreCDKey();

	void StartUpdate();
	void ContinueUpdate();
	bool FileHashMismatch(char *strFile, char *strHash);

	void UpdateCredits(double dTime);

	void UnlockAchievement(int iAchievement);

	char	m_strStoredLevel[32];
	double	m_dStartGameCountdown;

	xSceneGraph			 m_xSceneGraph;
	xGUI				 m_xGUI;
	xParser				 m_xParser;
	xAudioStream		*m_pAudioStream;

	// Game Update Helpers
	bool		 m_bUpdatingGame;
	xGUIText	*m_pUpdateWindowStatusText;
	xFileIO		 m_xUpdateBatchFile;
	int			 m_iUpdateFileListIndex;
	int			 m_iUpdateFileStartIndex;
	int			 m_iUpdateHashStartIndex;
	int			 m_iOutOfDateFiles;

	// Slide Transition Helpers
	bool		 m_bSlideTransitioning;
	double		 m_dSlideTransitionTime;
	bool		 m_bSlideTransitionRight;
	xGUIWindow	*m_pLeftWindow;
	xGUIWindow	*m_pRightWindow;

	// Drag and Drop Helpers
	xDragType	m_xDragType;
	int			m_iDragIndex;

	// Achievements
	char				 *m_strAchievementsPending[BB_PENDINGACHIEVEMENTS_MAX];
	int					  m_iAchievementsUnlocking[BB_PENDINGACHIEVEMENTS_MAX];
	DWORD				  m_dwAchievementUnlockStart[BB_PENDINGACHIEVEMENTS_MAX];
	xRenderable2DQuad	 *m_pAchievementIcon[BB_PENDINGACHIEVEMENTS_MAX];
	xRenderable2DText	  m_xAchievementTitle[BB_PENDINGACHIEVEMENTS_MAX];
	xTexture			**m_pAchievementIconTextures;

	// Challenge Helpers(NOTE: A lot of Free Play variables below are shared with Challenges)
	bool			 m_bChallenges;
	int				 m_iChallengeCount;
	int				 m_iChallengesTop;
	char			*m_strChallengeSongNames[BB_ACHIEVEMENTS_MAX];
	bool			 m_bChallengeLoopSong[BB_ACHIEVEMENTS_MAX];
	char			*m_strChallengeEnemies[BB_ACHIEVEMENTS_MAX];
	xGUIImage		*m_pChallengeCompleteImages[BB_FREEPLAY_SONGCOUNT + 1];
	xRenderable2D	 m_xChallengeCompleteOriginal[BB_FREEPLAY_SONGCOUNT + 2];
	xRenderable2D	 m_xChallengeCompleteStart[BB_FREEPLAY_SONGCOUNT + 1];
	xRenderable2D	 m_xChallengeCompleteTarget[BB_FREEPLAY_SONGCOUNT + 1];
	xGUIText		*m_pChallengeTextLines[4];
	xGUIImage		*m_pChallengeAchievementIcon;

	// Free Play Helpers
	int				  m_iFreePlaySongCount;
	char			**m_strFreePlaySongDirs;
	char			**m_strFreePlaySongList;
	char			**m_strFreePlayArtistList;
	int				 *m_iFreePlayDifficulties[3];
	char			**m_strFreePlayDefaultEnemies;
	int				  m_iFreePlayTopSong;
	int				  m_iFreePlayDifficulty;
	int				  m_iFreePlayExtraBlockCount;
	int				  m_iFreePlayEnemy;
	char			**m_strFreePlayEnemies;
	int				  m_iFreePlayEnemyCount;
	int				  m_iFreePlayScrollAmount; // Shared with Challenges
	float			  m_fFreePlayScrollTime; // Shared with Challenges
	float			  m_fFreePlayCurrentScrollTime; // Shared with Challenges
	int				  m_iFreePlayScrollDirection; // Shared with Challenges
	xGUIImage		 *m_pFreePlaySongBackgrounds[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongBGOriginal[BB_FREEPLAY_SONGCOUNT + 2]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongBGStart[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongBGTarget[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xGUIText		 *m_pFreePlaySongNames[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongNameOriginal[BB_FREEPLAY_SONGCOUNT + 2]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongNameStart[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xRenderable2D	  m_xFreePlaySongNameTarget[BB_FREEPLAY_SONGCOUNT + 1]; // Shared with Challenges
	xGUIText		 *m_pFreePlaySongArtist[2]; // Shared with Challenges
	int				  m_iCurrentArtistIndex; // Shared with Challenges
	float			  m_fFreePlaySongArtistOriginalY; // Shared with Challenges
	float			  m_fFreePlaySongArtistStartY[2]; // Shared with Challenges
	float			  m_fFreePlaySongArtistTargetY[2]; // Shared with Challenges
	xGUIImage		 *m_pFreePlaySongDiffStars[BB_FREEPLAY_SONGCOUNT + 1][12]; // Shared with Challenges
	float			  m_fFreePlaySongDiffOriginalY[BB_FREEPLAY_SONGCOUNT + 2]; // Shared with Challenges
	float			  m_fFreePlaySongDiffStartY[BB_FREEPLAY_SONGCOUNT + 2]; // Shared with Challenges
	float			  m_fFreePlaySongDiffTargetY[BB_FREEPLAY_SONGCOUNT + 2]; // Shared with Challenges
	xGUIModel		 *m_pFreePlayEnemyModel; // Shared with Challenges

	// Leaderboard Helpers
	int		m_iSPLeaderboardDifficulty;
	int		m_iSPLeaderboardStartIndex;
	float	m_fSPDifficultyOriginalY;

	// Multiplayer Helpers
	xGUIColumnListBox	*m_pServerListBox;
	xGUIButton			*m_pRefreshButton;
	bool				 m_bPingingServers;
	DWORD				*m_pPingTimes;
	int					 m_iPingTimesLength;
	xPingInfo			*m_pPingInfo;
	int					 m_iLastServerPinged;
	xGUIListBox			*m_pSongListBox;
	int					 m_iSongDifficulty;
	xHTTPQuery			 m_xHTTPQuery;

	// Temp Menu Variables
	float				 m_fKillMenuIntroTime;
	xPreloadGameOptions	 m_xPreloadGameOptions[BB_LEVELS_MAX + 1];
	xGUIImage			*m_pDifficultyArrow;
	int					 m_iSelectedDifficulty;
	int					 m_iTourIndex;
	int					 m_iTourBattleIndexStart;
	int					 m_iPlayerMoveCategory;
	int					 m_iBindingKeyIndex;
	xSinglePlayerLogin	 m_xSinglePlayerLogin;
	char				 m_strSPLoginTarget[32];
};

#endif