#include "peGame.h"

void xMultiplier::Init()
{
	Reset();

	if(!m_pMultiplierObject)
	{
		m_pMultiplierObject = xSceneGraphCreateQuad(BB_IMAGE_MULTIPLIERFILL, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pMultiplierObject, 4, BB_DEPTH_MULTIPLIER, 0, 1);
		xRenderable2DQuadSetPosition(m_pMultiplierObject, BB_FIELDX + BB_MULTIPLIER_LEFT, BB_FIELDY + BB_MULTIPLIER_TOP);

		m_pWhiteMultiplierObject = xSceneGraphCreateQuad(BB_IMAGE_WHITEMULTIPLIERFILL, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pWhiteMultiplierObject, 1, BB_DEPTH_WHITEMULTIPLIER, 0, 1);
		xRenderable2DQuadSetPosition(m_pWhiteMultiplierObject, BB_FIELDX + BB_MULTIPLIER_LEFT, BB_FIELDY + BB_MULTIPLIER_TOP);

		m_pWhiteBlockTimer = xSceneGraphCreateQuad(BB_IMAGE_BUILDOVERLAYWHITE, 0);
		xSceneGraphInsertQuad(&g_xGame.m_xSceneGraph, m_pWhiteBlockTimer, 4, BB_DEPTH_BUILDOVERLAYWHITE, 0, 1);
		xRenderable2DQuadSetPosition(m_pWhiteBlockTimer, BB_FIELDX + BB_BUILDOVERLAYWHITEX, BB_FIELDY + BB_BUILDOVERLAYWHITEY);
	}

	xRenderable2DQuadUpdateAlpha(m_pMultiplierObject, 1.0f);
	xRenderable2DQuad4UpdateRadialPercentage(m_pMultiplierObject, 0.0f);
	xRenderable2DQuadUpdateAlpha(m_pWhiteMultiplierObject, 0.0f);
	xRenderable2DQuadUpdate(m_pWhiteMultiplierObject);
	xRenderable2DQuad4UpdateRadialPercentage(m_pWhiteBlockTimer, 0.0f);
}

void xMultiplier::Update(double dTime)
{
	xColor tempcolor;
	float tempfloat;

	m_bMultiplierIncreased = 0;
	m_bMultiplierDecreased = 0;

	if(m_fMultiplierTimer > 0.0)
	{
		//DEBUGLOG("MultiplierUpdate - %i - %f - %f/%f\n", m_bNewMultiplierTime, m_fMultiplierAnimationTime, m_fOldMultiplierTime, m_fMultiplierTimer);

		if(m_bNewMultiplierTime)
		{
			m_fMultiplierAnimationTime += dTime;

			xRenderable2DQuad4UpdateRadialPercentage(m_pWhiteBlockTimer, (m_fOldMultiplierTime + ((m_fMultiplierTimer - m_fOldMultiplierTime) * (m_fMultiplierAnimationTime / 0.2f))) / m_fMaxMultiplierTime);

			if(m_fMultiplierAnimationTime >= 0.2f)			
				m_bNewMultiplierTime = 0;
		}
		else
		{
			m_fMultiplierTimer -= dTime;

			if(m_fMultiplierTimer < 0.0)
			{
				m_fMaxMultiplierTime = 0.0f;
				m_fMultiplierTimer = 0.0f;

				xRenderable2DQuad4UpdateRadialPercentage(m_pWhiteBlockTimer, 0.0f);
			}
			else
				xRenderable2DQuad4UpdateRadialPercentage(m_pWhiteBlockTimer, m_fMultiplierTimer / m_fMaxMultiplierTime);
		}
	}

	if(m_fMultiplierPercentage != m_fTargetMultiplierPercentage)
	{
		//DEBUGLOG("MultiplierUpdate: Percentage - %f/%f\n", m_fMultiplierPercentage, m_fTargetMultiplierPercentage);

		if(m_fMultiplierPercentage < m_fTargetMultiplierPercentage)
		{
			tempfloat = (m_fTargetMultiplierPercentage - m_fMultiplierPercentage) / 0.1f;

			if(tempfloat < 0.1f)
				tempfloat = 0.1f;

			m_fMultiplierPercentage += tempfloat * dTime / 1.5f;

			if(m_fMultiplierPercentage >= m_fTargetMultiplierPercentage)
			{
				if(m_fMultiplierPercentage >= 1.0f)
					m_fFadeToWhiteTime = 0.1f;

				m_fMultiplierPercentage = m_fTargetMultiplierPercentage;
			}
		}
		else if(m_fMultiplierPercentage > m_fTargetMultiplierPercentage)
		{
			m_fMultiplierPercentage -= (m_fMultiplierPercentage > 0.1f ? m_fMultiplierPercentage : 0.1f) * dTime * 10.0f;

			if(m_fMultiplierPercentage <= m_fTargetMultiplierPercentage)
				m_fMultiplierPercentage = m_fTargetMultiplierPercentage;
		}
		
		//DEBUGLOG("MultiplierUpdate: Percentage - Updating to %f\n", m_fMultiplierPercentage);
		xRenderable2DQuadUpdateAlpha(m_pMultiplierObject, 1.0f);
		xRenderable2DQuad4UpdateRadialPercentage(m_pMultiplierObject, m_fMultiplierPercentage);
	}
	else if(m_fFadeToWhiteTime > 0.0f)
	{
		m_fFadeToWhiteTime -= dTime;

		if(m_fFadeToWhiteTime <= 0.0f)
		{
			m_fFadeToWhiteTime = 0.0f;
			m_fFadeOutTime = 0.1f;
		}

		xRenderable2DQuadUpdateAlpha(m_pMultiplierObject, m_fFadeToWhiteTime / 0.1f);
		xRenderable2DQuadUpdateAlpha(m_pWhiteMultiplierObject, (0.1f - m_fFadeToWhiteTime) / 0.1f);
	}
	else if(m_fFadeOutTime > 0.0f)
	{
		m_fFadeOutTime -= dTime;

		if(m_fFadeOutTime <= 0.0f)
		{
			m_fFadeOutTime = 0.0f;
			m_fMultiplierPercentage = 0.0f;
			m_fTargetMultiplierPercentage -= 1.0f;
		}

		xRenderable2DQuadUpdateAlpha(m_pWhiteMultiplierObject, m_fFadeOutTime / 0.1f);
	}
}

void xMultiplier::RefreshAllRenderables()
{
	if(m_pMultiplierObject)
	{
		xRenderable2DQuadUpdateTexture(m_pMultiplierObject, m_pMultiplierObject->m_pTextures[0]);
		xRenderable2DQuad4UpdateRadialPercentage(m_pMultiplierObject, m_fMultiplierPercentage);

		xRenderable2DQuadUpdateTexture(m_pWhiteBlockTimer, m_pWhiteBlockTimer->m_pTextures[0]);
		xRenderable2DQuad4UpdateRadialPercentage(m_pWhiteBlockTimer, 0.0);

		xRenderable2DQuadUpdateTexture(m_pWhiteMultiplierObject, m_pWhiteMultiplierObject->m_pTextures[0]);
		xRenderable2DQuadUpdate(m_pWhiteMultiplierObject);
	}
}

void xMultiplier::MultiplierAddTime(float fAdditionalTime)
{
	m_bNewMultiplierTime = 1;
	m_fMultiplierAnimationTime = 0.0;
	m_fOldMultiplierTime = m_fMultiplierTimer;

	m_fMultiplierTimer += fAdditionalTime;

	if(m_fMultiplierTimer > m_fMaxMultiplierTime)
		m_fMaxMultiplierTime = m_fMultiplierTimer;
}
