#include "Paper.h"

// Support for grabbing MAC Address
#include <iphlpapi.h>
#pragma comment(lib, "IPHLPAPI.lib")

int xNetworkClient::Init(bool bTCP, int iWinProcMessage)
{
	WSADATA w;
	int iError = WSAStartup(0x0202, &w);

	if(iError)
		return PE_RES_WSINTERNALERROR;

	if(w.wVersion != 0x0202)
	{
		WSACleanup();
		return PE_RES_WSVERSIONERROR;
	}

	m_iWinProcMessage = iWinProcMessage;

	m_bTCP = bTCP;

	return PE_RES_NOERROR;
}

int xNetworkClient::Listen(int iPort)
{
	memset(&m_sockAddrIn, 0, sizeof(m_sockAddrIn));
	m_sockAddrIn.sin_family		 = AF_INET;
	m_sockAddrIn.sin_port		 = htons(iPort);
	m_sockAddrIn.sin_addr.s_addr = htonl(INADDR_ANY);

	if(m_bTCP)
	{
		if(!m_sockListen)
		{
			m_sockListen = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

			WSAAsyncSelect(m_sockListen, g_xPaperAPI.m_hWnd, m_iWinProcMessage, FD_WRITE | FD_READ | FD_CLOSE | FD_ACCEPT);

			if(bind(m_sockListen, (LPSOCKADDR)&m_sockAddrIn, sizeof(m_sockAddrIn)) == SOCKET_ERROR)
			{
				int iError = WSAGetLastError();
				WSACleanup();
				return PE_RES_WSINTERNALERROR;
			}

			if(listen(m_sockListen, 5) == SOCKET_ERROR)
			{
				WSACleanup();
				return PE_RES_WSINTERNALERROR;
			}
		}
	}
	else
	{
		if(m_bUDPClientBound)
			Close();

		m_sockConnection = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);

		WSAAsyncSelect(m_sockConnection, g_xPaperAPI.m_hWnd, m_iWinProcMessage, FD_WRITE | FD_READ | FD_CLOSE);

		if(bind(m_sockConnection, (LPSOCKADDR)&m_sockAddrIn, sizeof(m_sockAddrIn)) == SOCKET_ERROR)
		{
			int iError = WSAGetLastError();
			WSACleanup();
			return PE_RES_WSINTERNALERROR;
		}

		m_bUDPClientBound = true;
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Connect(const char* strIP, int iPort)
{
	if(!m_sockConnection)
	{
		if(m_bTCP)
			m_sockConnection = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		else
			m_sockConnection = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	}

	WSAAsyncSelect(m_sockConnection, g_xPaperAPI.m_hWnd, m_iWinProcMessage, FD_WRITE | FD_READ | FD_CLOSE | FD_CONNECT);

	memset(&m_sockAddrIn, 0, sizeof(m_sockAddrIn));
	m_sockAddrIn.sin_family		 = AF_INET;
	m_sockAddrIn.sin_port		 = htons(iPort);
	m_sockAddrIn.sin_addr.s_addr = inet_addr(strIP);

	if(m_bTCP)
	{
		if(connect(m_sockConnection, (LPSOCKADDR)&m_sockAddrIn, sizeof(m_sockAddrIn)) == SOCKET_ERROR)
		{
			int error = WSAGetLastError();
			if(error != 0 && error != 10035)
				return PE_RES_WSINTERNALERROR;
		}
	}
	else
	{
		if(m_bUDPClientBound)
			Close();

		sockaddr_in bindaddr = {0};
		bindaddr.sin_family		 = AF_INET;
		bindaddr.sin_port		 = htons(0);
		bindaddr.sin_addr.s_addr = htonl(INADDR_ANY);

		if(bind(m_sockConnection, (LPSOCKADDR)&bindaddr, sizeof(bindaddr)) == SOCKET_ERROR)
		{
			int iError = WSAGetLastError();
			WSACleanup();
			return PE_RES_WSINTERNALERROR;
		}

		m_bUDPClientBound = true;

		m_bConnected = true;
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Accept()
{
	if(!m_sockConnection)
	{
		int sizeaddr = sizeof(m_sockAddress);
		m_sockConnection = accept(m_sockListen, &m_sockAddress, &sizeaddr);
		if(m_sockConnection == INVALID_SOCKET)
			return PE_RES_WSINTERNALERROR;
	}
	else
	{
		sockaddr tempaddr;
		int sizeaddr = sizeof(tempaddr);
		SOCKET tempconnection = accept(m_sockListen, &tempaddr, &sizeaddr);
		closesocket(tempconnection);
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Read()
{
	if(m_bTCP)
	{
		char buffer[MAX_NETWORK_PACKETSIZE] = {0};

		int readsize = recv(m_sockConnection, buffer, MAX_NETWORK_PACKETSIZE, 0);

		if(readsize > 0)
		{
			memcpy(&m_strReceiveBuffer[m_iReadBufferOffset], buffer, readsize);

			m_iReadBufferOffset += readsize;
		}
	}
	else
	{
		xUDPPacketWinsock* packet = &m_xUDPPackets;

		while(packet->m_pNext)
			packet = packet->m_pNext;

		packet->m_pNext = new xUDPPacketWinsock;
		memset(packet->m_pNext->m_strData, 0, MAX_NETWORK_PACKETSIZE);

		int sockaddrsize = sizeof(packet->m_pNext->m_sockAddrIn);

		int readsize = recvfrom(m_sockConnection, packet->m_pNext->m_strData, MAX_NETWORK_PACKETSIZE, 0, (LPSOCKADDR)&packet->m_pNext->m_sockAddrIn, &sockaddrsize);

		if(readsize <= 0)
		{
			delete packet->m_pNext;
			packet->m_pNext = NULL;
		}
		else
		{
			packet = packet->m_pNext;
			packet->m_pNext = NULL;
			packet->m_dwTimeReceived = g_xPaperAPI.m_xPlatform.GetTime();
		}
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::Update()
{
	if(m_bTCP && m_iWriteBufferOffset > 0)
	{
		int sent = 0;
		if(m_iWriteBufferOffset >= MAX_NETWORK_PACKETSIZE)
		{
			int i = 0;
			char tmp[MAX_NETWORK_PACKETSIZE] = {0};
			for(; i < MAX_NETWORK_PACKETSIZE - 1; i++)
				tmp[i] = m_strSendBuffer[i];

			sent = send(m_sockConnection, tmp, MAX_NETWORK_PACKETSIZE, 0);

			if(sent == SOCKET_ERROR)
			{
				if(WSAGetLastError() == WSAEWOULDBLOCK)
					m_bWritable = false;
			}
			else if(sent > 0)
			{
				for(; i < m_iWriteBufferOffset; i++)
				{
					m_strSendBuffer[i - sent] = m_strSendBuffer[i];
					m_strSendBuffer[i] = 0;
				}

				m_iWriteBufferOffset -= sent;
			}
		}
		else
		{
			sent = send(m_sockConnection, m_strSendBuffer, m_iWriteBufferOffset, 0);

			if(sent == m_iWriteBufferOffset)
			{
				memset(m_strSendBuffer, 0, MAX_NETWORK_BUFFERSIZE);
				m_iWriteBufferOffset = 0;
			}
			else if(sent > 0)
			{
				for(int i = 0; i < m_iWriteBufferOffset; i++)
				{
					m_strSendBuffer[i - sent] = m_strSendBuffer[i];
					m_strSendBuffer[i] = 0;
				}

				m_iWriteBufferOffset -= sent;
			}
		}
	}

	return PE_RES_NOERROR;
}


void xNetworkClient::Close()
{
	if(m_sockConnection)
		closesocket(m_sockConnection);

	m_bConnected		= false;
	m_bWritable			= false;
	m_bUDPClientBound	= false;
	m_sockConnection	= NULL;
}

void xNetworkClient::Shutdown()
{
	//Send disconnect message?

	WSACleanup();
}

int xNetworkClient::Send(const char *strData, int iSize)
{
	int sent;

	if(m_bTCP)
	{
		memcpy(&m_strSendBuffer[m_iWriteBufferOffset], strData, iSize);

		m_iWriteBufferOffset += iSize;
	}
	else
	{
		sent = sendto(m_sockConnection, strData, iSize, 0, (LPSOCKADDR)&m_sockAddrIn, sizeof(m_sockAddrIn));
	}

	return PE_RES_NOERROR;
}

int xNetworkClient::SendResponse(const char *strData, int iSize)
{
	if(!m_bTCP)
	{
		sendto(m_sockConnection, strData, iSize, 0, (LPSOCKADDR)&m_xRespondAddress, sizeof(m_xRespondAddress));
	}

	return PE_RES_NOERROR;
}

char* xNetworkClient::Receive(char strBuffer[MAX_NETWORK_BUFFERSIZE + 1], DWORD &dwTimeReceived)
{
	if(m_bTCP)
	{
		memcpy(strBuffer, m_strReceiveBuffer, MAX_NETWORK_BUFFERSIZE);
		strBuffer[MAX_NETWORK_BUFFERSIZE] = '\0';

		memset(m_strReceiveBuffer, 0, MAX_NETWORK_BUFFERSIZE);

		m_iReadBufferOffset = 0;
	}
	else
	{
		xUDPPacketWinsock* packet = m_xUDPPackets.m_pNext;
		if(packet)
		{
			m_xUDPPackets.m_pNext = packet->m_pNext;

			memcpy(strBuffer, packet->m_strData, MAX_NETWORK_PACKETSIZE);

			dwTimeReceived = packet->m_dwTimeReceived;

			m_xRespondAddress = packet->m_sockAddrIn;

			delete packet;
		}
		else
		{
			return NULL;
		}
	}

	return strBuffer;
}

void xNetworkClient::GetMacAddress(uint8_t *pMAC)
{
	IP_ADAPTER_ADDRESSES adapters[64];
	PIP_ADAPTER_ADDRESSES adapter;
	DWORD status, buflen = sizeof(adapters);

	if((status = GetAdaptersAddresses(AF_UNSPEC, 0, 0, adapters, &buflen)) != ERROR_SUCCESS)
	{
		memset(pMAC, 0, 8);
		return;
	}

	for(adapter = adapters; adapter; adapter = adapter->Next)
	{
		if(adapter->PhysicalAddressLength > 0 && ((adapter->FirstDnsServerAddress && adapter->OperStatus == IfOperStatusUp) ||
			!memcmp(pMAC, adapter->PhysicalAddress, 8)) && (adapter->IfType == IF_TYPE_ETHERNET_CSMACD || adapter->IfType == IF_TYPE_IEEE80211) )
		{
			memset(pMAC, 0, 8);
			memcpy(pMAC, adapter->PhysicalAddress, adapter->PhysicalAddressLength);
			return;
		}
	}

	memset(pMAC, 0, 8);
}