#include "Paper.h"
#include "resource.h"

int xPlatform::Init()
{
	return PE_RES_NOERROR;
}

HWND xPlatform::MakeWindow(HINSTANCE hInstance, void *pWinProc, int iWindowWidth, int iWindowHeight, int iBPP, bool bFullscreen)
{
	WNDCLASSEX winclass; // Using extend version of windows class
	
	//Set the App's HWND to null so we can make sure it worked later
	HWND hwnd = NULL;

	winclass.cbSize			= sizeof(WNDCLASSEX);
	winclass.hInstance		= hInstance;
	winclass.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;						//Redraw On Size, And Own DC For Window.
	winclass.cbClsExtra		= 0;														//Specifies the number of extra bytes to allocate following the window-class structure
	winclass.cbWndExtra		= 0;														//Specifies the number of extra bytes to allocate following the window instance
	winclass.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));			//LoadIcon(hinstance, MAKEINTRESOURCE(PE_ICON));
	winclass.hIconSm		= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ICON1));
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	winclass.lpszMenuName	= NULL;
	winclass.hbrBackground	= NULL;
	winclass.lpszClassName	= PE_CLASSNAME;												//The app's class name
	winclass.lpfnWndProc	= (long (__stdcall *)(struct HWND__ *,unsigned int,unsigned int,long))pWinProc;

	//We must register the class with windows
	if(!m_bRegisteredClass && !RegisterClassEx(&winclass))
		return false;

	m_bRegisteredClass = true;

	//And then we create the window
	if(bFullscreen)
	{
		m_hWnd = hwnd = CreateWindowEx(WS_EX_APPWINDOW, PE_CLASSNAME, PE_CLASSNAME, WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE | WS_POPUP,
							  0, 0, iWindowWidth, iWindowHeight, NULL, NULL, hInstance, NULL);

	}
	else
	{
		RECT winrect = {0, 0, iWindowWidth, iWindowHeight};
		AdjustWindowRectEx(&winrect, WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX & ~WS_THICKFRAME, FALSE, WS_EX_APPWINDOW | WS_EX_WINDOWEDGE);
		m_hWnd = hwnd = CreateWindowEx(WS_EX_APPWINDOW | WS_EX_WINDOWEDGE, PE_CLASSNAME, PE_CLASSNAME,
							  WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_VISIBLE | WS_OVERLAPPEDWINDOW & ~WS_MAXIMIZEBOX & ~WS_THICKFRAME, 0, 0, 
							  winrect.right - winrect.left, winrect.bottom - winrect.top, NULL, NULL, 
							  hInstance, NULL);

		m_iBorderSize		= ((winrect.right  - winrect.left) - (iWindowWidth)) / 2 - 1;
		m_iTitleBarHeight	= (winrect.bottom - winrect.top ) - (iWindowHeight + m_iBorderSize);
	}

	// Draw the window
	ShowWindow(hwnd, SW_NORMAL);
	UpdateWindow(hwnd);

	return hwnd;
}

void xPlatform::GetDateAndTime(int &iDateYear, int &iDateMonth, int &iDateDay, int &iTimeHour, int &iTimeMinutes, int &iTimeSeconds)
{
	time_t longseconds;
	tm *timestruct;

	time(&longseconds);
	timestruct = gmtime(&longseconds);

	iDateYear	= timestruct->tm_year;
	iDateMonth	= timestruct->tm_mon;
	iDateDay	= timestruct->tm_mday;
	iTimeHour	= timestruct->tm_hour;
	iTimeMinutes= timestruct->tm_min;
	iTimeSeconds= timestruct->tm_sec;
}

char* xPlatform::GetClipboardContents(char *strOut, int iMaxLength)
{
	HANDLE clip;
	int i;

	strOut[0] = '\0';

	if(OpenClipboard(NULL))
	{
		clip = GetClipboardData(CF_TEXT);

		if(clip)
		{
			for(i = 0; i < iMaxLength; i++)
			{
				strOut[i] = ((char*)clip)[i];

				if(strOut[i] == '\0')
					break;
			}

			strOut[iMaxLength - 1] = '\0';
		}

		CloseClipboard();
	}

	return strOut;
}

void xPlatform::Shutdown()
{
}
