/////////////////////////////////////////////////////////////////
// Name: xSceneGraph.h
/////////////////////////////////////////////////////////////////
// Description:
// - Interface for creating/ordering game objects
/////////////////////////////////////////////////////////////////
// - 14.12.2010, Created, Dayle Flowers (Xienen)
//   Created as new interface for creating game objects
/////////////////////////////////////////////////////////////////
#ifndef XSCENEGRAPH_H_
#define XSCENEGRAPH_H_

#include "xEngineDefines.h"
#include "xRenderable2D.h"
#include "xRenderable2DLine.h"
#include "xRenderable2DBox.h"
#include "xRenderable2DQuad.h"
#include "xRenderable3D.h"

struct xSceneGraph
{
	xRenderDataShader	m_xRenderData;

	#ifdef _DEBUG
		#define MAX_RENDERABLE2DS	1024
		xRenderable2D *Renderable2DArray[MAX_RENDERABLE2DS];
	#endif
};

FORCE_INLINE void xSceneGraphInit(xSceneGraph *pThis)
{
	xRenderDataShaderInit(&pThis->m_xRenderData);

	#ifdef _DEBUG
		memset(pThis->Renderable2DArray, 0, sizeof(xRenderable2D*) * MAX_RENDERABLE2DS);
	#endif
}

void xSceneGraphDepthChanged(xSceneGraph *pThis, xRenderDataVerts *pVertData, float fDepth);
void xSceneGraphRemove2D(xSceneGraph *pThis, xRenderable2D *pRenderable);

xRenderable2DLine* xSceneGraphCreateLine();
void xSceneGraphInsertLine(xSceneGraph *pThis, xRenderable2DLine *pLine, float fDepth);
void xSceneGraphDeleteLine(xSceneGraph *pThis, xRenderable2DLine *pLine);

xRenderable2DBox* xSceneGraphCreateBox();
void xSceneGraphInsertBox(xSceneGraph *pThis, xRenderable2DBox *pBox, float fDepth);
void xSceneGraphDeleteBox(xSceneGraph *pThis, xRenderable2DBox *pBox);

xRenderable2DQuad* xSceneGraphCreateQuad(const char* strFilePath, int iFrame);
xRenderable2DQuad* xSceneGraphCreateQuad(xTexture *pTexture);
xRenderable2DQuad* xSceneGraphCreateQuad(xColor *pColor);
void xSceneGraphInsertQuad(xSceneGraph *pThis, xRenderable2DQuad *pQuad, int iQuadCount, float fDepth, bool bColorArray = 0, bool bForceUniqueTextureData = 0);
void xSceneGraphDeleteQuad(xSceneGraph *pThis, xRenderable2DQuad *pQuad, bool bDeleteTexture);
void xSceneGraphQuadTextureChanged(xSceneGraph *pThis, xRenderable2DQuad *pQuad);

xRenderable3D* xSceneGraphCreateMesh(const char* strTexturePath, int iFrame);
xRenderable3D* xSceneGraphCreateMesh(xTexture *pTexture);
void xSceneGraphInsertMesh(xSceneGraph *pThis, xRenderable3D *pRenderable, int iTriangleCount, float fDepth, bool bForceUniqueTextureData = 0);
void xSceneGraphDeleteMesh(xSceneGraph *pThis, xRenderable3D *pRenderable, bool bDeleteTexture);

void xSceneGraphRemoveAllRenderables(xSceneGraph *pThis);

#ifdef _DEBUG
	#define SCENEGRAPH_DEBUG_ADD_QUADS(x,y,z)	xSceneGraphVerifyAddQuads(x,y,z)
	void xSceneGraphVerifyAddQuads(xSceneGraph *pThis, xRenderable2DQuad *pQuads, int iCount);

	#define SCENEGRAPH_DEBUG_REMOVE_QUADS(x,y,z)	xSceneGraphVerifyRemoveQuads(x,y,z)
	void xSceneGraphVerifyRemoveQuads(xSceneGraph *pThis, xRenderable2DQuad *pQuads, int iCount);
#else
	#define SCENEGRAPH_DEBUG_ADD_QUADS(x,y,z)		0
	#define SCENEGRAPH_DEBUG_REMOVE_QUADS(x,y,z)	0
#endif

#endif