#include "Paper.h"

int xSound::Open(const char *strPathName)
{
	if(LoadWave(strPathName, 0))
		return PE_RES_OPENALMUSICLOADFILEERROR;

	m_xSoundState = PE_SOUNDSTATE_LOADED;
	m_iNumSounds = 1;

	m_uiSource = PE_SOUNDSOURCE_NONE;

	return PE_RES_NOERROR;
}

int xSound::AddVariant(const char *strPathName)
{
	if(m_iNumSounds >= MAX_VARIANTS || LoadWave(strPathName, m_iNumSounds))
		return PE_RES_OPENALMUSICLOADFILEERROR;

	m_iNumSounds++;

	return PE_RES_NOERROR;
}

int xSound::Play(float fVolume, bool bLooping, int iPan)
{
	int index, i;
	bool alreadyplayed;

	//DEBUGLOG("xSound.Play - OldState=%i - %s\n", m_xSoundState, m_strObjectPath);

	// Grab an available source
	m_uiSource = g_xPaperAPI.m_xAudio.GetSource(false);

	// Select a sound
	if(m_bDontRepeat)
	{
		do
		{
			alreadyplayed = false;
			index = rand() % m_iNumSounds;

			for(i = 0; i < m_iCurrentPlayedIndex; i++)
			{
				if(m_pPlayedSounds[i] == index)
				{
					alreadyplayed = true;
					break;
				}
			}
		} while(alreadyplayed);

		m_pPlayedSounds[m_iCurrentPlayedIndex++] = index;

		if(m_iCurrentPlayedIndex >= m_iNumSounds)
			m_iCurrentPlayedIndex = 0;
	}
	else
		index = rand() % m_iNumSounds;

	// Attach Source to Buffer and Play
	//DEBUGLOG("xSound.Play - OldState=%i - %s - %u - %i/%i - %u\n", m_xSoundState, m_strObjectPath, m_uiSource, index, m_iNumSounds, m_uiBuffers[index]);
	alSourcef(m_uiSource, AL_GAIN, fVolume);
	alSourcei(m_uiSource, AL_LOOPING, bLooping ? AL_TRUE : AL_FALSE);
	alSourcei(m_uiSource, AL_BUFFER, m_uiBuffers[index]);
	alSourcePlay(m_uiSource);

	m_xSoundState = PE_SOUNDSTATE_PLAYING;

	return PE_RES_NOERROR;
}

int xSound::Stop()
{
	alSourceStop(m_uiSource);

	m_xSoundState = PE_SOUNDSTATE_STOPPED;

	return PE_RES_NOERROR;
}

int xSound::Pause()
{
	m_xSoundState = PE_SOUNDSTATE_PAUSED;
	alSourcePause(m_uiSource);
	return PE_RES_NOERROR;
}

int xSound::UnPause()
{
	m_xSoundState = PE_SOUNDSTATE_PLAYING;
    alSourcePlay(m_uiSource);
	return PE_RES_NOERROR;
}

void xSound::Release()
{
	alDeleteBuffers(m_iNumSounds, m_uiBuffers);
}

int xSound::LoadWave(const char *strPathName, int iBufferIndex)
{
	xFileIO file;

	if(file.OpenFile(strPathName, true, true))
		return PE_RES_OPENALMUSICLOADFILEERROR;

	// Generate an AL Buffer
	alGenBuffers(1, &m_uiBuffers[iBufferIndex]);

	// Load Wave file into OpenAL Buffer
	ALint			iDataSize, iFrequency;
	ALenum			bufferformat = 0;
	WAVEFILEINFO	waveinfo;
	WAVEFILEHEADER	wavefileheader;
	RIFFCHUNK		riffchunk;
	WAVEFMT			waveformat;

	memset(&waveinfo, 0, sizeof(WAVEFILEINFO));

	// Read Wave file header
	file.ReadData(&wavefileheader, sizeof(WAVEFILEHEADER));

	if(_strnicmp(wavefileheader.szRIFF, "RIFF", 4) || _strnicmp(wavefileheader.szWAVE, "WAVE", 4))
	{
		file.CloseFile();
		return PE_RES_OPENALMUSICLOADFILEERROR;
	}

	while(!file.ReadData(&riffchunk, sizeof(RIFFCHUNK)))
	{
		if(!_strnicmp(riffchunk.szChunkName, "fmt ", 4))
		{
			if(riffchunk.ulChunkSize <= sizeof(WAVEFMT))
			{
				file.ReadData(&waveformat, riffchunk.ulChunkSize);
							
				// Determine if this is a WAVEFORMATEX or WAVEFORMATEXTENSIBLE wave file
				if (waveformat.usFormatTag == WAVE_FORMAT_PCM)
				{
					waveinfo.wfType = WF_EX;
					memcpy(&waveinfo.wfEXT.Format, &waveformat, sizeof(PCMWAVEFORMAT));
				}
				else if (waveformat.usFormatTag == WAVE_FORMAT_EXTENSIBLE)
				{
					waveinfo.wfType = WF_EXT;
					memcpy(&waveinfo.wfEXT, &waveformat, sizeof(WAVEFORMATEXTENSIBLE));
				}
			}
			else
				file.SkipAhead(riffchunk.ulChunkSize);
		}
		else if(!_strnicmp(riffchunk.szChunkName, "data", 4))
		{
			waveinfo.ulDataSize = riffchunk.ulChunkSize;
			waveinfo.ulDataOffset = file.GetCurrentOffset();
			file.SkipAhead(riffchunk.ulChunkSize);
		}
		else
			file.SkipAhead(riffchunk.ulChunkSize);

		// Ensure that we are correctly aligned for next chunk
		if(riffchunk.ulChunkSize & 1)
			file.SkipAhead(1);
	}

	if(!waveinfo.ulDataSize || !waveinfo.ulDataOffset || (waveinfo.wfType != WF_EX && waveinfo.wfType != WF_EXT))
	{
		file.CloseFile();
		return PE_RES_OPENALMUSICLOADFILEERROR;
	}

	// Allocate memory for sample data
	waveinfo.pData = new char[waveinfo.ulDataSize];

	// Seek to start of audio data
	file.SetPosition(waveinfo.ulDataOffset);

	// Read Sample Data
	if(file.ReadData(waveinfo.pData, waveinfo.ulDataSize))
	{
		delete [] waveinfo.pData;
		file.CloseFile();
		return PE_RES_OPENALMUSICLOADFILEERROR;
	}

	// Determine the OpenAL Buffer Format
	if(waveinfo.wfType == WF_EX)
	{
		if(waveinfo.wfEXT.Format.nChannels == 1)
		{
			switch(waveinfo.wfEXT.Format.wBitsPerSample)
			{
				case 4:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO_IMA4");
					break;
				case 8:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO8");
					break;
				case 16:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO16");
					break;
			}
		}
		else if(waveinfo.wfEXT.Format.nChannels == 2)
		{
			switch(waveinfo.wfEXT.Format.wBitsPerSample)
			{
				case 4:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO_IMA4");
					break;
				case 8:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO8");
					break;
				case 16:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO16");
					break;
			}
		}
		else if(waveinfo.wfEXT.Format.nChannels == 4 && waveinfo.wfEXT.Format.wBitsPerSample == 16)
			bufferformat = alGetEnumValue("AL_FORMAT_QUAD16");
	}
	else if(waveinfo.wfType == WF_EXT)
	{
		if(waveinfo.wfEXT.Format.nChannels == 1 &&
		   (waveinfo.wfEXT.dwChannelMask == SPEAKER_FRONT_CENTER ||
			waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT) ||
			waveinfo.wfEXT.dwChannelMask == 0))
		{
			switch(waveinfo.wfEXT.Format.wBitsPerSample)
			{
				case 4:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO_IMA4");
					break;
				case 8:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO8");
					break;
				case 16:
					bufferformat = alGetEnumValue("AL_FORMAT_MONO16");
					break;
			}
		}
		else if(waveinfo.wfEXT.Format.nChannels == 2 && waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT))
		{
			switch(waveinfo.wfEXT.Format.wBitsPerSample)
			{
				case 4:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO_IMA4");
					break;
				case 8:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO8");
					break;
				case 16:
					bufferformat = alGetEnumValue("AL_FORMAT_STEREO16");
					break;
			}
		}
		else if(waveinfo.wfEXT.Format.nChannels == 2 && waveinfo.wfEXT.Format.wBitsPerSample == 16 &&
				waveinfo.wfEXT.dwChannelMask == (SPEAKER_BACK_LEFT|SPEAKER_BACK_RIGHT))
		{
			bufferformat =  alGetEnumValue("AL_FORMAT_REAR16");
		}
		else if(waveinfo.wfEXT.Format.nChannels == 4 && waveinfo.wfEXT.Format.wBitsPerSample == 16 &&
				waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT|SPEAKER_BACK_LEFT|SPEAKER_BACK_RIGHT))
		{
			bufferformat = alGetEnumValue("AL_FORMAT_QUAD16");
		}
		else if(waveinfo.wfEXT.Format.nChannels == 6 && waveinfo.wfEXT.Format.wBitsPerSample == 16 &&
				waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT|SPEAKER_FRONT_CENTER|SPEAKER_LOW_FREQUENCY|SPEAKER_BACK_LEFT|SPEAKER_BACK_RIGHT))
		{
			bufferformat = alGetEnumValue("AL_FORMAT_51CHN16");
		}
		else if(waveinfo.wfEXT.Format.nChannels == 7 && waveinfo.wfEXT.Format.wBitsPerSample == 16 &&
				waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT|SPEAKER_FRONT_CENTER|SPEAKER_LOW_FREQUENCY|SPEAKER_BACK_LEFT|SPEAKER_BACK_RIGHT|SPEAKER_BACK_CENTER))
		{
			bufferformat = alGetEnumValue("AL_FORMAT_61CHN16");
		}
		else if(waveinfo.wfEXT.Format.nChannels == 8 && waveinfo.wfEXT.Format.wBitsPerSample == 16 &&
				waveinfo.wfEXT.dwChannelMask == (SPEAKER_FRONT_LEFT|SPEAKER_FRONT_RIGHT|SPEAKER_FRONT_CENTER|SPEAKER_LOW_FREQUENCY|SPEAKER_BACK_LEFT|SPEAKER_BACK_RIGHT|SPEAKER_SIDE_LEFT|SPEAKER_SIDE_RIGHT))
		{
			bufferformat = alGetEnumValue("AL_FORMAT_71CHN16");
		}
	}

	if(bufferformat == 0)
	{
		delete [] waveinfo.pData;
		file.CloseFile();
		return PE_RES_OPENALMUSICLOADFILEERROR;
	}

	// Add the data to a Buffer
	alBufferData(m_uiBuffers[iBufferIndex], bufferformat, waveinfo.pData, waveinfo.ulDataSize, waveinfo.wfEXT.Format.nSamplesPerSec);

	delete [] waveinfo.pData;
	file.CloseFile();

	return PE_RES_NOERROR;
}