#ifndef XSOUND_H_
#define XSOUND_H_

#include <ks.h>
#include <ksmedia.h>
#include "openal/al.h"
#include "openal/alc.h"
#include "xAudio.h"

#define MAX_VARIANTS	8

enum xSoundState
{
	PE_SOUNDSTATE_NULL = 0,
	PE_SOUNDSTATE_LOADED,
	PE_SOUNDSTATE_STOPPED,
	PE_SOUNDSTATE_PLAYING,
	PE_SOUNDSTATE_PAUSED
};

enum xPanSetting
{
	PE_SOUNDPAN_FULL_LEFT = 0,
	PE_SOUNDPAN_CENTERED = 128,
	PE_SOUNDPAN_FULL_RIGHT = 255,
	PE_SOUNDPAN_FULL_VOLUME = 256
};

class xSound : public xPoolItem<xSound>
{
public:
	xSound()
	{
		m_iNumSounds	= 0;
		m_bDontRepeat	= 0;
	}

	int  Open(const char *strPathName);
	int  AddVariant(const char *strPathName);
	int  Play(float fVolume = 1.0, bool bLooping = false, int iPan = PE_SOUNDPAN_FULL_VOLUME);
	int  Stop();
	int  Pause();
	int  UnPause();
	void Release();
	bool IsPlaying() { ALint state; if(m_xSoundState != PE_SOUNDSTATE_PLAYING) return false; alGetSourcei(m_uiSource, AL_SOURCE_STATE, &state); if(state != AL_PLAYING) { m_xSoundState = PE_SOUNDSTATE_STOPPED; return false; } return true; }
	bool IsPaused() { return m_xSoundState == PE_SOUNDSTATE_PAUSED; }

	// OpenAL Function Pointers
	LPALGETENUMVALUE			alGetEnumValue;
	LPALGENBUFFERS				alGenBuffers;
	LPALGENSOURCES				alGenSources;
	LPALGETSOURCEI				alGetSourcei;
	LPALBUFFERDATA				alBufferData;
	LPALSOURCEI					alSourcei;
	LPALSOURCEF					alSourcef;
	LPALSOURCEPLAY				alSourcePlay;
	LPALSOURCESTOP				alSourceStop;
	LPALSOURCEPAUSE				alSourcePause;
	LPALDELETESOURCES			alDeleteSources;
	LPALDELETEBUFFERS			alDeleteBuffers;

	xSoundState m_xSoundState;
	ALuint		m_uiSource;
	int			m_iNumSounds;

	bool		 m_bDontRepeat;
	int			*m_pPlayedSounds;
	int			 m_iCurrentPlayedIndex;

protected:
	int  LoadWave(const char *strPathName, int iBufferIndex);

	ALuint	m_uiBuffers[MAX_VARIANTS];
};

enum WAVEFILETYPE
{
	WF_EX  = 1,
	WF_EXT = 2
};

#ifndef _WAVEFORMATEX_
#define _WAVEFORMATEX_
typedef struct tWAVEFORMATEX
{
    WORD    wFormatTag;
    WORD    nChannels;
    DWORD   nSamplesPerSec;
    DWORD   nAvgBytesPerSec;
    WORD    nBlockAlign;
    WORD    wBitsPerSample;
    WORD    cbSize;
} WAVEFORMATEX;
#endif

#ifndef _WAVEFORMATEXTENSIBLE_
#define _WAVEFORMATEXTENSIBLE_
typedef struct
{
    WAVEFORMATEX    Format;
    union
	{
        WORD wValidBitsPerSample;       /* bits of precision  */
        WORD wSamplesPerBlock;          /* valid if wBitsPerSample==0 */
        WORD wReserved;                 /* If neither applies, set to zero. */
    } Samples;
    DWORD           dwChannelMask;      /* which channels are present in stream  */
    GUID            SubFormat;
} WAVEFORMATEXTENSIBLE, *PWAVEFORMATEXTENSIBLE;
#endif

struct WAVEFILEINFO
{
	WAVEFILETYPE	wfType;
	WAVEFORMATEXTENSIBLE wfEXT;		// For non-WAVEFORMATEXTENSIBLE wavefiles, the header is stored in the Format member of wfEXT
	char			*pData;
	unsigned long	ulDataSize;
	unsigned long	ulDataOffset;
};

#pragma pack(push, 4)

struct WAVEFILEHEADER
{
	char			szRIFF[4];
	unsigned long	ulRIFFSize;
	char			szWAVE[4];
};

struct RIFFCHUNK
{
	char			szChunkName[4];
	unsigned long	ulChunkSize;
};

struct WAVEFMT
{
	unsigned short	usFormatTag;
	unsigned short	usChannels;
	unsigned long	ulSamplesPerSec;
	unsigned long	ulAvgBytesPerSec;
	unsigned short	usBlockAlign;
	unsigned short	usBitsPerSample;
	unsigned short	usSize;
	unsigned short  usReserved;
	unsigned long	ulChannelMask;
    GUID            guidSubFormat;
};

#pragma pack(pop)

#endif