#include "Paper.h"

FORCE_INLINE void xTextureEnsurePowerOf2Size(xTexture *pThis, int iDataWidth, int iDataHeight, int *pOutWidth, int *pOutHeight, unsigned char* &pInOutData)
{
	unsigned char *newdata;
	int iOutWidth = 2;
	int iOutHeight = 2;
	int heightdiff, i;

	while(iOutWidth < iDataWidth)
		iOutWidth <<= 1;

	while(iOutHeight < iDataHeight)
		iOutHeight <<= 1;

	if(iDataWidth == iOutWidth && iDataHeight == iOutHeight)
	{
		*pOutWidth = iOutWidth;
		*pOutHeight = iOutHeight;
		return;
	}

	newdata = (unsigned char*)dlmalloc(sizeof(unsigned char) * iOutWidth * iOutHeight * 4);
	memset(newdata, 0, iOutWidth * iOutHeight * 4);

	heightdiff = iOutHeight - iDataHeight;
	for(i = iDataHeight - 1; i >= 0; i--)
		memcpy(&newdata[(i + heightdiff) * iOutWidth * 4], &pInOutData[i * iDataWidth * 4], iDataWidth * 4);

	dlfree(pInOutData);

	*pOutWidth = iOutWidth;
	*pOutHeight = iOutHeight;
	pInOutData = newdata;
}

int xTextureCreateTexture(xTexture *pThis, xFileIO* pTextureFile, int iWidth, int iHeight)
{
	unsigned char *ucdata;

	pThis->m_bHasAlpha	= false;
	pThis->m_iWidth = iWidth;
	pThis->m_iHeight = iHeight;

	ucdata = (unsigned char*)dlmalloc(sizeof(unsigned char) * iWidth * iHeight * 4);

	pTextureFile->ReadData(ucdata, iWidth * iHeight * 4);

	xTextureEnsurePowerOf2Size(pThis, iWidth, iHeight, &pThis->m_iDataWidth, &pThis->m_iDataHeight, ucdata);

	//We don't use multiple of 2 textures, we use multiple of 1
	//NOTE: This was intended to fix lack of 1 to 1 for textures, but it didn't seem to do anything
	//glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//Generate a texture with the associative texture ID stored in the array
	//if(pThis->m_uiID != 0)
	//{
	//	glDeleteTextures(1, &pThis->m_uiID);
	//	glFlush();
	//}

	glGenTextures(1, &pThis->m_uiID);

	//Bind the texture to the texture arrays index and init the texture
	glBindTexture(GL_TEXTURE_2D, pThis->m_uiID);

	//Lastly, we need to tell OpenGL the quality of our texture map.  GL_LINEAR_MIPMAP_NEAREST
	//is the smoothest.  GL_LINEAR_MIPMAP_LINEAR is faster than GL_LINEAR_MIPMAP_NEAREST, 
	//but looks blotchy and pixelated.  Good for slower computers though. 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

	//Clamp texture to edge of quad
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, pThis->m_iDataWidth, pThis->m_iDataHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, ucdata);

	dlfree(ucdata);

	return PE_RES_NOERROR;
}

int xTextureSkipTexture(xFileIO *pTextureFile, int iWidth, int iHeight)
{
	pTextureFile->SkipAhead(iWidth * iHeight * 4);

	return PE_RES_NOERROR;
}

/*int xTexture::CreateStretchedTexture(xFileIO* pTextureFile, int iDataWidth, int iDataHeight, int iDesiredWidth, int iDesiredHeight)
{
	m_iWidth  = iDesiredWidth;
	m_iHeight = iDesiredHeight;

	//Read image data from file
	unsigned char *ucdata = new unsigned char[iDataWidth * iDataHeight * 4];
	pTextureFile->ReadData(ucdata, iDataWidth * iDataHeight * 4);

	//Turn the image data into Stretched Data
	unsigned char *stretcheddata = new unsigned char[m_iWidth * m_iHeight * 4];
	int cornersize = (iDataWidth < iDataHeight ? (m_iWidth < iDataWidth / 2 ? m_iWidth : iDataWidth) : (m_iHeight < iDataWidth / 2 ? m_iHeight : iDataHeight)) / 4;

	//Copy corners
	for(int y = 0; y < cornersize; y++)
	{
		//Lower Left
		memcpy(stretcheddata + m_iWidth * y * 4, ucdata + iDataWidth * y * 4, cornersize * 4);

		//Lower Right
		memcpy(stretcheddata + m_iWidth * y * 4 + (m_iWidth - cornersize) * 4, ucdata + iDataWidth * y * 4 + (iDataWidth - cornersize) * 4, cornersize * 4);

		//Upper Left
		memcpy(stretcheddata + m_iWidth * (m_iHeight - y - 1) * 4, ucdata + iDataWidth * (iDataHeight - y - 1) * 4, cornersize * 4);

		//Upper Right
		memcpy(stretcheddata + m_iWidth * (m_iHeight - y - 1) * 4 + (m_iWidth - cornersize) * 4, ucdata + iDataWidth * (iDataHeight - y - 1) * 4 + (iDataWidth - cornersize) * 4, cornersize * 4);
	}

	//Fill in bottom edge
	for(int y = 0; y < cornersize; y++)
	{
		int x;
		for(x = cornersize; x < m_iWidth - (cornersize * 2); x += cornersize)
			memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * y * 4 + cornersize * 4, cornersize * 4);

		int remainingwidth = m_iWidth - x - cornersize;
		memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * y * 4 + cornersize * 4, remainingwidth * 4);
	}

	//Fill in left and right edges, as well as center
	for(int y = cornersize; y < m_iHeight - cornersize; y++)
	{
		//Copy left edge
		memcpy(stretcheddata + m_iWidth * y * 4, ucdata + iDataWidth * ((y % cornersize) + cornersize) * 4, cornersize * 4);

		// Fill the center
		int x;
		for(x = cornersize; x < m_iWidth - (cornersize * 2); x += cornersize)
			memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * ((y % cornersize) + cornersize) * 4 + cornersize * 4, cornersize * 4);

		int remainingwidth = m_iWidth - x - cornersize;
		memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * ((y % cornersize) + cornersize) * 4 + cornersize * 4, remainingwidth * 4);

		//Copy right edge
		memcpy(stretcheddata + m_iWidth * y * 4 + (m_iWidth - cornersize) * 4, ucdata + iDataWidth * ((y % cornersize) + cornersize) * 4 + (iDataWidth - cornersize) * 4, cornersize * 4);
	}

	//Fill in upper edge
	for(int y = m_iHeight - cornersize, datay = iDataHeight - cornersize; y < m_iHeight; y++, datay++)
	{
		int x;
		for(x = cornersize; x < m_iWidth - (cornersize * 2); x += cornersize)
			memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * datay * 4 + cornersize * 4, cornersize * 4);

		int remainingwidth = m_iWidth - x - cornersize;
		memcpy(stretcheddata + m_iWidth * y * 4 + x * 4, ucdata + iDataWidth * datay * 4 + cornersize * 4, remainingwidth * 4);
	}

	EnsurePowerOf2Size(m_iWidth, m_iHeight, m_iDataWidth, m_iDataHeight, stretcheddata);

	//We don't use multiple of 2 textures, we use multiple of 1
	//NOTE: This was intended to fix lack of 1 to 1 for textures, but it didn't seem to do anything
	//glPixelStorei(GL_UNPACK_ALIGNMENT, 1);

	//Generate a texture with the associative texture ID stored in the array
	glGenTextures(1, &m_uiID);

	//Bind the texture to the texture arrays index and init the texture
	glBindTexture(GL_TEXTURE_2D, m_uiID);

	//Lastly, we need to tell OpenGL the quality of our texture map.  GL_LINEAR_MIPMAP_NEAREST
	//is the smoothest.  GL_LINEAR_MIPMAP_LINEAR is faster than GL_LINEAR_MIPMAP_NEAREST, 
	//but looks blotchy and pixelated.  Good for slower computers though. 
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

	//Clamp texture to edge of quad
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_EDGE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);

	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, m_iDataWidth, m_iDataHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, stretcheddata);

	delete [] ucdata;
	delete [] stretcheddata;

	return PE_RES_NOERROR;
}
*/
